#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "lib.h"
#include "internet.h"
#include "upgrade.h"
#include "comm.h"

#define COMPORT				COM2
#define DIRECTORY_LIST      "dir.txt"


#if OPH|PHL|PHL1000|PHL2700
#define ABORT_KEY			CLR_KEY
#else
#define ABORT_KEY			UP_KEY
#endif

// Structure with all communication settings
ftp_settings ftp_setup;

//Global function
int Transmit_and_Update(const char *filename, const char *version);

//Local functions
static int SetISP(struct isp_config *isp);
static int SetFTP(const char *filename, struct ftp_config *ftp, struct dns_config *dns);
static void process_error(int error);
static int WaitForCradle(void);
static int _atoi(char *str);


/**************************************************************************************/
/* Transmit_and_Update                                                                */
/**************************************************************************************/
/* Description                                                                        */
/*   Transmit a data file to an FTP and searches for a software update.               */
/*   If a software update was found it is downloaded and installed automatically      */
/*                                                                                    */
/* Input                                                                              */
/*   char *filename   :  File name of the data that is transmitted to the FTP server  */
/*   char *version    :  Current 5 char version string (without software code)        */
/*                       (i.e. 4100A in case the software version is CFX4000A)        */
/* Input (Global)                                                                     */
/* ftp_settings ftp_setup : struct with all communication settings stored in strings  */
/*                                                                                    */
/* Returns                                                                            */
/*      0           :   Succes													      */
/*     -1           :   Not all parameters have been set                              */
/*     -2           :   User aborted					                              */
/*     -3           :   Communication error											  */
/**************************************************************************************/
int Transmit_and_Update(const char *filename, const char *version)
{
	static struct isp_config isp;
	static struct ftp_config ftp;
	static struct dns_config dns;
	static char update_file[13];

	int ret, error;

	// Display Progress bars only
	TCPIP_init(3, 0);			
	
	// Parse all ISP related settings
	if(SetISP(&isp) < 0)
		return -1;

	// Parse all FTP related settings
	if(SetFTP(filename, &ftp, &dns) < 0)
		return -1;

	resetkey();

	if(isp.com == COM2)			// Cradle
	{
		if(WaitForCradle() < 0)	// Wait for terminal to be placed in its cradle
			return -2;
	}
	else if(isp.com == COM1)	// IrDA (or Cradle)
	{
		printf("\f\tr Communication \tr\nPress any key\nto start");	
		while(!kbhit())
			idle();
		resetkey();
	}

#if OPH|PHL|PHL1000|PHL2700
#else
	if(ischarging())
		flipscreen(ON);
#endif
	
	printf("\f\tr Communication \tr\nDialing...\n");
	
	// Dial ISP
	if( (error = connect(&isp)) != 0 )
		goto tcpip_error;

	// If Domain name of FTP was specified, search for DNS server and get IP-address of FTP-server
	if( ftp_setup.ftp_domain[0] != '\0')
	{
		// Check if IPCP protocol retreives any DNS servers 
		if( (error = get_ISP_server_IP(&dns, 30)) != 0 )
			goto tcpip_error;

		
		printf("\f\tr Communication \tr\nSearching host\n");

		if(get_host_by_name(&dns))		// get IP-address of FTP-server
		{
			if( ftp_setup.ftp_ip[0] == '\0' )
				goto tcpip_error;
		}
		else
		{
			printf("IP-addr found:\n%d.%d.%d.%d", dns.IP1, dns.IP2, dns.IP3, dns.IP4);
			delay(25);

			ftp.IP1 = dns.IP1;			// set IP-address of FTP-server
			ftp.IP2 = dns.IP2;
			ftp.IP3 = dns.IP3;
			ftp.IP4 = dns.IP4;
		}
	}

	printf("\f\tr Communication \tr\nConnect to FTP\n");
	
	// Connect and login at the FTP server
	if ((error = ftp_command(&ftp, FTP_INIT)) != 0) 
		goto tcpip_error;

	// Check if their is data to send, if so send it to the FTP server
	if(fsize((char *)filename) > 0)	
	{
		printf("\f\tr Communication \tr\nSending file..\n");
											
		if ((error = ftp_command(&ftp, FTP_PUT)) != 0) 
			goto tcpip_error;
	}

	// Get the directory list of the FTP-server (DIR *.*)
	strcpy((char *)ftp.local_file, DIRECTORY_LIST);
	ftp.remote_file[0] = '\0';		// No mask
	ftp.append = FALSE;			// Overwrite any existing file on the terminal						

	printf("\f\tr Communication \tr\nList directory\n");
	
	if ((error = ftp_command(&ftp, FTP_LIST)) != 0) 
		goto tcpip_error;

	// Check for software updates in the received directory list
	ret = CheckForUpdates(DIRECTORY_LIST, version, update_file);

	switch(ret)
	{
		case CURRENT:	// Software is up to date
			printf("\nSoftware is\nup to date!\n%s", update_file);
			break;
		case LOWER:		// Lower software version found
			printf("\nOlder version\nfound.\n%s", update_file);
			break;
		case HIGHER:	// Higher software version found
			printf("\nNewer version\nfound.\n%s", update_file);
			break;
		case NONE:		// No software versions found at all
		default:
			printf("\nNo updates\nfound!");
			break;
	}

	remove(DIRECTORY_LIST);
	delay(50);

	// Newer version found -> Start downloading update
	if(ret == HIGHER)	
	{
		strcpy((char *)ftp.local_file, update_file);
		strcpy((char *)ftp.remote_file, update_file);

		printf("\f\tr Communication \tr\nRetreiving appl\n");
		
		if ((error = ftp_command(&ftp, FTP_GET)) != 0) 
			goto tcpip_error;
	}

	// Close FTP communication
	printf("\f\tr Communication \tr\nClosing FTP...");
	
	if ((error = ftp_command(&ftp, FTP_QUIT)) != 0) 
		goto tcpip_error;

	// Disconnect from ISP and hang up modem
	disconnect(&isp);

	if(fsize((char *)filename) > 0)		// Delete transmitted data
		remove(filename);

	// If new software found & downloaded -> Install update
	if(ret == HIGHER)
	{
		printf("\fPlease wait...\nNew application\nis being\ninstalled...");

		if(runapplfromfile(update_file) < 0)
			printf("\fApplication\ncould not be\ninstalled!");
	}

	printf("\nReady!\nPress any key");

#if OPH|PHL|PHL1000|PHL2700
#else
	flipscreen(OFF);
#endif

	return 0;	// Done!
	
tcpip_error:	// Error handling
	
	if(ftp_connected())
		ftp_command(&ftp, FTP_QUIT);

	disconnect(&isp);

#if OPH|PHL|PHL1000|PHL2700
#else
	flipscreen(OFF);
#endif

	process_error(error);

	return -3;
}

/**************************************************************************************/
/* SetISP                                                                             */
/**************************************************************************************/
/* Description                                                                        */
/*   Parsed the ISP settings from 'ftp_setup' to the isp_config structure that's to   */
/*   be used by the TCP/IP library                                                    */
/*                                                                                    */
/* Input (Global)                                                                     */
/* ftp_settings ftp_setup : struct with all communication settings stored in strings  */
/*                                                                                    */
/* Output                                                                             */
/*   struct isp_config *isp :  structure in which parsed ISP settings are stored	  */
/*                                                                                    */
/* Returns                                                                            */
/*      0           :   Succes													      */
/*     -1           :   Not all parameters have been set                              */
/**************************************************************************************/
int SetISP(struct isp_config *isp)
{
	if( ftp_setup.isp_nr[0] == '\0')
		return -1;

	strcpy((char *)isp->phonenumber, ftp_setup.isp_nr);

	if( ftp_setup.isp_user[0] == '\0')
		return -1;

	strcpy((char *)isp->user, ftp_setup.isp_user);

	if( ftp_setup.isp_pass[0] == '\0')
		return -1;

	strcpy((char *)isp->pass, ftp_setup.isp_pass);

	isp->com = isdigit(ftp_setup.isp_com[0]) ? (ftp_setup.isp_com[0]-'0') : COMPORT;
	isp->abort_key = ABORT_KEY;

	return 0;
}

/**************************************************************************************/
/* SetFTP                                                                             */
/**************************************************************************************/
/* Description                                                                        */
/*   Parsed the FTP settings from 'ftp_setup' to the ftp_config and dns_config        */
/*   structure that are to be used by the TCP/IP library                              */
/*                                                                                    */
/* Input                                                                              */
/*   char *filename   :  File name of the data that is transmitted to the FTP server  */
/*                                                                                    */
/* Input (Global)                                                                     */
/* ftp_settings ftp_setup : struct with all communication settings stored in strings  */
/*                                                                                    */
/* Output                                                                             */
/*   struct ftp_config *ftp :  structure in which the parsed FTP settings are stored  */
/*   struct dns_config *dns :  struct in which the domain name of FTP server is stored*/
/*                                                                                    */
/* Returns                                                                            */
/*      0           :   Succes													      */
/*     -1           :   Not all parameters have been set                              */
/**************************************************************************************/
int SetFTP(const char *filename, struct ftp_config *ftp, struct dns_config *dns)
{
	char *ptr;

	if( ftp_setup.ftp_domain[0] == '\0' && ftp_setup.ftp_ip[0] == '\0')
		return -1;

	if( ftp_setup.ftp_user[0] == '\0')
		return -1;

	if( ftp_setup.ftp_pass[0] == '\0')
		return -1;

	// If set, store the domain name of the FTP-server in the DNS structure
	if( ftp_setup.ftp_domain[0] != '\0')
	{
		strcpy((char *)dns->domain_address, ftp_setup.ftp_domain);
		dns->abort_key = ABORT_KEY;
	}
	
	// If set, parse the IP-address with format -> XXX.XXX.XXX.XXX
	if( ftp_setup.ftp_ip[0] != '\0')	
	{
		ptr = ftp_setup.ftp_ip;

		ftp->IP1 = _atoi(ptr);
		ptr = strchr(ptr, '.');
		ftp->IP2 = _atoi(++ptr);
		ptr = strchr(ptr, '.');
		ftp->IP3 = _atoi(++ptr);
		ptr = strchr(ptr, '.');
		ftp->IP4 = _atoi(++ptr);
	}

	strcpy((char *)ftp->user, ftp_setup.ftp_user);		// Username
	strcpy((char *)ftp->pass, ftp_setup.ftp_pass);		// Password
	
	ftp->append = FALSE;						// Overwrite any existing file on the FTP-server						
	ftp->abort_key = ABORT_KEY;

	strcpy((char *)ftp->local_file, filename);			// File name of the data that is transmitted to the FTP server
	strcpy((char *)ftp->remote_file, filename);

	return OK;
}

/**************************************************************************************/
/* process_error                                                                      */
/**************************************************************************************/
/* Description                                                                        */
/*   Error handling of returned value by the TCP/IP library                           */
/*	                                                                                  */
/* Input                                                                              */
/*   int error  :  Error value as defined in 'internet.h'                             */
/**************************************************************************************/
void process_error(int error)
{
	printf("\f\tr Communication \tr\nError occured:\n");
	
	switch(error)
	{
		case ERR_USER_ABORTED:				printf("Aborted by user");					break;
		case ERR_NO_SUCH_FILE:				printf("Can't find or\nopen file");			break;
		case ERR_POLLED:					printf("Server time out");					break;
		case ERR_ILLEGAL_COMMAND:			printf("Command rejected\nby server");		break;
		case ERR_SERVER_REJECTED:			printf("Data rejected\nby server");			break;
		case ERR_CONNECTION_ABORTED:		printf("Connection\naborted");				break;
		case ERR_TCP_PORT_NOT_SUPPORTED:	printf("Invalid TCP/IP\nport number");		break;
		case ERR_TCP_RECEIVED_RESET:		printf("Connection\nis reset");				break;
		case ERR_DIALUP_COM_PORT:			printf("Comport error");					break;
		case ERR_NODIALTONE:				printf("No dial tone");						break;
		case ERR_BUSY:						printf("Line busy");						break;
		case ERR_NOANSWER:					printf("Server does not\nrespond");			break;
		case ERR_NOCARRIER:					printf("No carrier");						break;
		case ERR_MODEM:						printf("Modem Error");						break;
		case ERR_DELAYED:					printf("Modem 'Delayed'\nerror");			break;
		case ERR_BLACKLISTED:				printf("Phone nr. is\nblack listed");		break;
		case ERR_NORESPONSE:				printf("Modem doesn't\nrespond");			break;
		case ERR_BUFFER:					printf("Receive buffer\noverflow");			break;
		case ERR_CLOSING_COM_PORT:			printf("Error closing\nCOM port");			break;
		case ERR_LOGIN_FAIL:				printf("Login at ISP\nfailed");				break;
		case ERR_OFF_HOOK:					printf("Modem off hook");					break;
		case ERR_FTP_TEMP_ERROR:			printf("FTP Server\nwon't receive");		break;
		case ERR_FTP_NOT_LOGGED_IN:			printf("Login at FTP\nfailed");				break;
		case ERR_FTP_SERVER_DOWN:			printf("FTP Server\ndoesn't respond");		break;
		case ERR_FTP_SYNTAX:				printf("FTP communi-\ncation error");		break;
		case ERR_FTP_NO_CONNECTION:			printf("Not connected\nto FTP");			break;
		case ERR_DNS_NO_RESPONSE:			printf("DNS servers\ndon't respond");		break;
		case ERR_DNS_DATA_INVALID:			printf("DNS response\nerror");				break;
		case ERR_DNS_NO_SERVERS_FOUND:		printf("No DNS servers\nfound");			break;
		case ERR_DNS_NAME_ERROR	:			printf("DNS error->\nHost not found");		break;
		default:							printf("Error #%d",error);					break;
	}
}

/**************************************************************************************/
/* WaitForCradle                                                                      */
/**************************************************************************************/
/* Description                                                                        */
/*   Waits for the terminal to be placed in the cradle                                */
/*	                                                                                  */
/* Returns                                                                            */
/*      -1    :  Abort key was pressed                                                */
/*       0    :  Terminal was placed in its cradle                                    */
/**************************************************************************************/
int WaitForCradle(void)
{
	if(!ischarging()) 
	{
		printf("\f\tr Communication \tr\nPlace terminal\nin the cradle");
	
		while (!ischarging()) 
		{
			idle();

			if(kbhit())
			{	if(getchar() == ABORT_KEY)
				{	resetkey();
					return -1;
				}
			}
		}
	}
	
	return 0;
}


/**************************************************************************************/
/* _atoi                                                                              */
/**************************************************************************************/
/* Description                                                                        */
/*   Convert strings to an integer                                      .             */
/**************************************************************************************/
int _atoi(char *str)
{
	int ch;
	int temp;

	temp = 0;
	while ((ch = *str++)!=0) {
		if (isspace(ch))
			continue;
		if (!isdigit(ch))
			break;
		temp *= 10;
		temp += ch - '0';
	}
	return (temp);
}
