#include <stdio.h>
#include <string.h>
#include "lib.h"
#include "upgrade.h"

// File extensions for different types of terminals
#define OPL_FILE_EXT         ".S32"
#define PHL_FILE_EXT         ".S2"
#define OPH_FILE_EXT         ".HEX"

// First character of the software code of different types of terminals
#define OPL_VERSION_CHAR     'L'
#define PHL_VERSION_CHAR     'C'
#define OPH_VERSION_CHAR     'X'

// Public functions
int CheckForUpdates(const char *fname_dir, const char *version, char *fresult);

// Local functions
static signed char find_application_version(const char *fname, const char *target, char *fversion);
static void StringToLower(char *str);

/**************************************************************************************/
/* CheckForUpdates                                                                    */
/**************************************************************************************/
/* Description                                                                        */
/*   Searches for a software update in a file with an FTP directory listing           */
/*                                                                                    */
/* Input                                                                              */
/*   char *fname_dir :  File name with FTP directory listing                          */
/*   char *version   :  Current 5 char version string (without software code)         */
/*                       (i.e. 4000A if the software version is CFX4000A)             */
/*                                                                                    */
/* Output                                                                             */
/*   char *fresult   :  If present,filename of latest version found in the directory  */
/*                       (Mininum size of string: 13)                                 */
/* Returns                                                                            */
/*   NONE   (-1)     :   No software versions found at all                            */
/*   SAME	(0)      :   Software is up to date                                       */
/*   LOWER	(1)      :   Lower software version found                                 */
/*   HIGHER	(2)      :   Higher software version found                                */
/**************************************************************************************/
int CheckForUpdates(const char *fname_dir, const char *version, char *fresult)
{
	static char current_version[8+1];

	signed char ret;

	version_os(current_version);
	current_version[1] = 'F';
	current_version[3] = '\0';
	strcat(current_version, version);
	current_version[7] = '\0';			//remove last character of version

	//Is there an application file present ?
	if( (ret = find_application_version(fname_dir, (const char *)current_version, fresult)) <= 0)
		return NONE;

	if( (char)ret == version[4] )
		return CURRENT;

	if( (char)ret < version[4] )
		return LOWER;

	return HIGHER;
}

/**************************************************************************************/
/* find_application_version                                                           */
/**************************************************************************************/
/* Description                                                                        */
/*   Searches for a software update in a file with an FTP directory listing           */
/*                                                                                    */
/* Input                                                                              */
/*   char *fname      :  File name with FTP directory listing                         */
/*   char *target     :  version string (without last modification characters)        */
/*                                                                                    */
/* Output                                                                             */
/*   char *version    :  highest version present                                      */
/*                                                                                    */
/* Returns                                                                            */
/*     -1           :   File (with FTP directory listing) not found                   */
/*      0           :   No software versions found                                    */
/*     >0           :   Software version found, returns last character of version     */
/**************************************************************************************/
signed char find_application_version(const char *fname, const char *target, char *fversion)
{
	FILE *fp;
	char *ptr;
	char string[80 + 1];
	char file_extension[1+3+1] = {0};
	char file_extension_l[1+3+1] = {0};
	signed char target_found;

	// Get the file extension for the specific terminal type
	switch(target[0])
	{
		case OPL_VERSION_CHAR:
			strcpy(file_extension, OPL_FILE_EXT);	// PHL series
			break;
		case PHL_VERSION_CHAR:
			strcpy(file_extension, PHL_FILE_EXT);	// OPL series
			break;
		case OPH_VERSION_CHAR:
			strcpy(file_extension, OPH_FILE_EXT);	// OPH series
			break;
		default:
			file_extension[0] = '\0';				// Unknown terminal, don't check file extension
	}

	strcpy(file_extension_l, file_extension);
	StringToLower(file_extension_l);				// File extension (lower case)
	//

	target_found = 0;

	if ((fp = fopen(fname, "rb")) == NULL)			// Open directory listing
		return (-2);

	while (fgets(string, 80, fp) != NULL)
	{
		if ((ptr = strchr(string, '\r')) != NULL)
			*ptr = '\0';

		if ( (ptr=strstr(string, target)) != NULL) 	/* found target file in directory */
		{
			if( strncmp(&ptr[8], file_extension, strlen(file_extension)) != 0 )		// Check file extension (Uppper case)
			{
				if( strncmp(&ptr[8], file_extension_l, strlen(file_extension)) != 0 )		// Check file extension (Lower case)
					continue;
			}

			if( target_found < ptr[7] )	// Check if new version is higher than previous one
			{
				target_found = ptr[7];

				// Copy version in result string
				strncpy(fversion, ptr, 8 + strlen(file_extension));
				fversion[8 + strlen(file_extension)] = '\0';

			}
		}
	}
	fclose(fp);

	return target_found;
}

/**************************************************************************************/
/* StringToLower                                                                      */
/**************************************************************************************/
/* Description                                                                        */
/*   Converst a string to lower case                                                  */
/*                                                                                    */
/* Input/Output                                                                       */
/*   char *str  :  String that is converted to lower case                             */
/**************************************************************************************/
void StringToLower(char *str)
{
	int i;

	for(i=0;i<strlen(str);++i)
	{
		if(str[i] >= 'A' && str[i] <= 'Z')
			str[i] += 0x20;
	}
}
