//------------------------------------------------------------------
//  Demo application for the OPL-9714/5
//------------------------------------------------------------------
//  This demo demonstrates how the GPS module of the OPL9714/5
//  can be used to collect GPS data.  Any retrieved GPRS-information
//  is displayed on the screen and NMEA messages from the GPS module
//  are transmitted by IrDA (cradle) at 115200 bps for debug
//  purposes.
//------------------------------------------------------------------
//  Version history:
//     LBW35450		:	First release
//                      Minimal required OS version: LBWV0313
//						Release date: December 15th, 2009
//						Author: Rens Dol
//						Company: Opticon Sensors Europe BV
//------------------------------------------------------------------
#include "lib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//
// Defines
//
#define GPS_BUF_LEN  200
#define GPS_TIMEOUT  ( 4000 / 20 )
#define GPS_APD_TIME ( 10000/ 20 )

//
// Public functions
//
void pos2dms(const char gps[10], const char nesw, unsigned int* deg, unsigned int* min, unsigned int* sec, unsigned int* msec);
char* get_field(char* buff, int field, char* result, int max_len);
void handle_gpgga(char* buff);
void handle_gprmc(char* buff);
void handle_gpgsa(char* buff);
void handle_gpgsv(char* buff);
void handle_mnea_sentence(char* gps_buff);
unsigned char itoh(unsigned int c);
int get_nmea_sentence(char* gps_buffer, int buf_size);


//------------------------------------------------------------------
// pos2dms
// ============================
// Converts the position string that was returned by the GPS module 
// into GPS coordinates (deg, minutes, secodes and msec.)
//
// gps : position string that was returned by the GPS module 
// nesw : Contains 'N' (north) 'E' (east) 'S' (south) or 'W' (west)
//
// Returns GPS coordinate:
//		deg	:	degrees
//		min :	minutes
//		sec :	seconds
//		msec :	msec
//------------------------------------------------------------------
void pos2dms(const char gps[10], const char nesw, unsigned int* deg, unsigned int* min, unsigned int* sec, unsigned int* msec)
{
	unsigned int d=0, m=0, s=0;
	int ret;

	if(nesw == 'N' || nesw == 'S')
	{
		ret = sscanf( gps, "%2u%2u.%u", &d, &m, &s );
	}
	else
	{
		ret = sscanf( gps, "%3u%2u.%u", &d, &m, &s );
	}

	if( ret == 3 )
	{
		*deg = d;
		*min = m;
		*sec = (s*6) / 1000;
		*msec = (s*6) % 1000;
	}
}


//------------------------------------------------------------------
// get_field
// ============================
// Parses a comma separated string and returns the content of the
// requested field number.
//
// *buff   : Comma separated input buffer
// field   : field number of which the content should be returned.
// result  : Pointer to the result buffer in which the requested field is copied
// max_len : Max. length of the result buffer
//
// Returns : Pointer that points to to the start of the specified field  
//---------------------------------------------------------------------
char* get_field( char* buff, int field, char* result , int max_len )
{
	char* 	ptr;
	char* 	ptr_end;
	int 	len;

	ptr = buff;
	while( field-- && ptr != NULL  )
	{
		ptr = strchr(ptr+1, ',');
	}

	if( result )
	{
		ptr_end = strchr(ptr+1, ',');
		len = (int)(ptr_end-ptr-1);
		len = (len<max_len) ? len : max_len;
		strncpy( result, ptr+1, len );
		result[len] = '\0';
	}
	return ptr+1;
}


//------------------------------------------------------------------
// handle_gpgga
// ============================-------------------------------------
// GGA - essential GPS fix data which provides 3D location and accuracy data.
//
// Example data:
// 
// $GPGGA,123519,4807.038,N,01131.000,E,1,08,0.9,545.4,M,46.9,M,,*47
//
// Where:
//     GGA          Global Positioning System Fix Data
//     123519       Fix taken at 12:35:19 UTC
//     4807.038     Latitude 48 deg 07.038'
//     N 			North / South
//     01131.000	Longitude 11 deg 31.000'
//     E  		    East / West
//     1            Fix quality: 
//                      0 = invalid
//                      1 = GPS fix (SPS)
//                      2 = DGPS fix
//                      3 = PPS fix
//			       		4 = Real Time Kinematic
//			            5 = Float RTK
//				        6 = estimated (dead reckoning) (2.3 feature)
//			            7 = Manual input mode
//			            8 = Simulation mode
//     08           Number of satellites being tracked
//     0.9          Horizontal dilution of position
//     545.4,M      Altitude, Meters, above mean sea level
//     46.9,M       Height of geoid (mean sea level) above WGS84
//                      ellipsoid
//     (empty field) time in seconds since last DGPS update
//     (empty field) DGPS station ID number
//     *47          the checksum data, always begins with *
//---------------------------------------------------------------------
void handle_gpgga( char* buff )
{
	char sat[5] = {0};
	char dop[5] = {0};

	putstring(buff );
	get_field(buff, 7, sat, 5);
	get_field(buff, 8, dop, 5);

	gotoxy(0, 1);
	printf("sat=%2.2s DOP=%3.3s", sat, dop);
}

//------------------------------------------------------------------
// handle_gprmc
// ============================-------------------------------------
// Parses a Recommended Minimum Specific GPS Transit Data (RMC) 
// sentence and displays it on the screen.
//
// Example data:
// 
// $GPRMC,123519,A,4807.038,N,01131.000,E,022.4,084.4,230394,003.1,W*6A
//
//Where:
//     RMC          Recommended Minimum sentence C
//     123519       Fix taken at 12:35:19 UTC
//     A            Status A=active or V=Void.
//     4807.0380,N  Latitude 48 deg 07.0380' N
//     01131.0000,E Longitude 11 deg 31.0000' E
//     022.4        Speed over the ground in knots
//     084.4        Track angle in degrees True
//     230394       Date - 23rd of March 1994
//     003.1,W      Magnetic Variation
//     *6A          The checksum data, always begins with *
//---------------------------------------------------------------------
void handle_gprmc( char* buff )
{
	char data[25] = { '0' };
	char nesw[3] = { '0' };
	unsigned int deg=0, min=0, sec=0, msec=0;

	putstring(buff);

	get_field(buff, 7, data, 25);
	gotoxy(0, 2);
	printf("Speed %s k/s", data);

	get_field(buff, 8, data, 25);
	gotoxy(0, 3);
	printf("Hdg %s", data);

	get_field(buff, 3, data, 25);
	get_field(buff, 4, nesw, 25);

	gotoxy(0, 4);
	pos2dms(data, nesw[0], &deg, &min,&sec, &msec);
	printf("%s%u%c%02u'%02u.%03u\"", nesw, deg, 0xF8, min, sec, msec);

	get_field( buff, 5, data, 25);
	get_field( buff, 6, nesw, 25);

	gotoxy(0,5);
	pos2dms(data, nesw[0], &deg, &min ,&sec, &msec);
	printf("%s%u%c%02u'%02u.%03u\"", nesw, deg, 0xF8, min, sec, msec);
}

//------------------------------------------------------------------
// handle_gpgsa
// ============================-------------------------------------
// Parses a GSA string and displays it on the screen.
// GSA = GPS Satellite status
//
// Example data:
// 
// $GPGSA,A,3,04,05,,09,12,,,24,,,,,2.5,1.3,2.1*39
//
// Where:
//     GSA      Satellite status
//     A        Auto selection of 2D or 3D fix (M = manual)
//     3        3D fix - values include: 1 = no fix
//                                       2 = 2D fix
//                                       3 = 3D fix
//     04,05... PRNs of satellites used for fix (space for 12)
//     2.5      PDOP (dilution of precision)
//     1.3      Horizontal dilution of precision (HDOP)
//     2.1      Vertical dilution of precision (VDOP)
//     *39      the checksum data, always begins with *
//---------------------------------------------------------------------
void handle_gpgsa( char* buff )
{
	char data[25] = {0};
	const char* const fix[] ={ "-", "No", "2D", "3D" };
	putstring( buff );

	get_field(buff, 2, data, 25);
	gotoxy(0, 0);
	printf("Fix = %2.2s" , fix[data[0]-'0']);
}

//------------------------------------------------------------------
// handle_gpgsv
// ============================-------------------------------------
// Outputs a GSV string 
//
// GSV string contain the number of SVs in view, PRN numbers, 
// elevation, azimuth & SNR values.
//
// Example data:
// 
// $GPGSV,2,1,08,01,40,083,46,02,17,308,41,12,07,344,39,14,22,228,45*75
//
//  Where:
//      GSV          Satellites in view
//      2            Number of sentences for full data
//      1            sentence 1 of 2
//      08           Number of satellites in view
//
//      01           Satellite PRN number
//      40           Elevation, degrees
//      083          Azimuth, degrees
//      46           SNR - higher is better for up to 4 satellites per sentence
//      *75          the checksum data, always begins with *
//---------------------------------------------------------------------
void handle_gpgsv( char* buff )
{
	putstring( buff );
}

//------------------------------------------------------------------
// handle_mnea_sentence
// ============================-------------------------------------
// Handles mnea sentences by handling the following types of nmean
// sentences
//
// GPGSV :	GSV string contains the number of SVs in view, PRN numbers, 
//			elevation, azimuth & SNR values. 
// GPGGA :  GGA string contains essential GPS fix data which 
//			provides 3D location and accuracy data.
// GPRMC :  RMC string contains Rcommended Minimum Specific GPS Transit Data
// GPGSA :  GSA string contains GPS Satellite status
//---------------------------------------------------------------------
void handle_mnea_sentence( char* gps_buff )
{
	typedef struct
	{
		const char* sentence;		// Compare string
		void (*fn)( char* );		// Function pointer
	}nmea_handler_t;

	const nmea_handler_t mnea_sentence_handler[]=			// Contains the various types of mnea scentence and the
	{														// corresponding parser functions that need to be called.
		{"GPGSV", handle_gpgsv },
		{"GPGGA", handle_gpgga },
		{"GPRMC", handle_gprmc },
		{"GPGSA", handle_gpgsa },
	};

	int i;

	for(i=0; i < ( sizeof(mnea_sentence_handler) / sizeof(nmea_handler_t) ); i++ )
	{
		if( strncmp( mnea_sentence_handler[i].sentence, gps_buff + 1, strlen( mnea_sentence_handler[i].sentence ) ) == 0 )
		{
			mnea_sentence_handler[i].fn( gps_buff );
		}
	}
}

//------------------------------------------------------------------
// itoh
// ============================-------------------------------------
// Converst an int between 0 and 15 to hexidecimal readable character
//
// Returns:
//		'0'..'F'	: On values 0...15
//		-1			: On value out of range
//---------------------------------------------------------------------
unsigned char itoh(unsigned int c)
{
	const char hex[]= "0123456789ABCDEF";
	
	if( c < 16 )
		return hex[ c ];

	return (unsigned char)(-1);
}

//------------------------------------------------------------------
// get_nmea_sentence
// ============================-------------------------------------
// Handles the receiving of GPS messages from the module
//
// Each GPS message starts with a '$' and ends with an '*' followed
// by a 2 digiti checksum and an additional "\r\n" .
//
// buf_size contains the size of gps_buffer
//
// The received GPS message is returned in gps_buffer
//
// Returns
//		OK	:	On success
//      <0	:	On error
//---------------------------------------------------------------------
int get_nmea_sentence( char* gps_buffer, int buf_size )
{
	int result = ERROR;
	int ch;
	static int state = 0;
	static int ix;
	static unsigned char sum;
	static unsigned int check;

	ch = ReadGps();
	if( ch != -1 )
	{
		switch( state )
		{
			case 0:						// wait for start of sentence ('$')
				if( ch == '$' )
				{
					ix = sum =0;
					gps_buffer[ix++] = (char)ch;
					state = 1;
				}
				break;

			case 1:				// Receiving data
				if( ix < buf_size  )
				{
					if( ch != '*' )		// End of message starts with '*'
					{
						sum ^= (unsigned char)ch;
					}
					else
					{
						state = 2;
					}
					gps_buffer[ix++] = (char)ch;
				}
				else
				{
					result = ERROR - state;
					ix = 0;
					state = 0;
				}
				break;

			case 2:						// Receive and check 1st byte of checksum
				gps_buffer[ix++] = (char)ch;
				if( ch == itoh(  ( sum >> 4 ) & 0x0F ) )
				{
					state++;
				}
				else
				{
					state  = 0; // checksum error;
					result = ERROR - state;
				}
				break;

			case 3:						// Receive and check 2nd byte of checksum
				gps_buffer[ix++] = (char)ch;
				if( ch == itoh( sum & 0x0F ) )
				{
					state++;
				}
				else
				{
					state = 0; 	// checksum error;
					result = ERROR - state;
				}
				break;

			case 4:						// line end (\r\n)
				if( ch == 0x0A  &&  gps_buffer[ ix-1 ] == 0x0D )
				{
					gps_buffer[ ix++ ] = (char)ch;
					gps_buffer[ ix++ ] = '\0';
					result = OK;
					state = 0;
				}
				else if( ix < buf_size )
				{
					gps_buffer[ ix++ ] = (char)ch;
				}
				else
				{
					state = 0;
				}
				break;

			default:					// undefined state
				state = 0;
				break;
		}
	}
	return result;
}

//------------------------------------------------------------------
// main
// ============================-------------------------------------
// Main function of the demo applicaiton
//
// Initializes the barcode engine, GPS module and serial port
// Receives
// by a 2 digiti checksum and an additional "\r\n" .
//
// buf_size contains the size of gps_buffer
//
// The received GPS message is returned in gps_buffer
//
// Returns
//		OK	:	On success
//      <0	:	On error
//---------------------------------------------------------------------
void main(void)
{
	static char gps_buff[GPS_BUF_LEN];
    char bcr_buf[42] = {0};
    struct barcode code  = {0};

    code.min   = 3;
    code.max   = 41;
    code.text  = bcr_buf; 

	scannerpower(SINGLE, 50*3 );		// Enable barcode reading: Single read / 2 seconds

	setfont( SMALL_FONT , NULL );
	
	InitGps();							// Initialize GPS and turn on GPS module
	GpsPower( ON );

	systemsetting( "SZ/R" );			// SZ (Baudrate: 115200) ; /R (Enable battery low indicator)

	comopen( COM1 );					// Open IrDA port

	starttimer( GPS_TIMEOUT  );
	printf( "\f");

	for(;;)
	{
		if( readbarcode(&code) == OK )	// Check if a barcode has been read
		{
			goodreadled(GREEN, 10);
			sound( TSTANDARD, VHIGH, SMEDIUM, SHIGH, 0);
			scannerpower(SINGLE, 500 / 20 );
		}

		if( endtimer() )				// If no valid nmea senteces have been received
		{								
			printf( "\f\n\tr NO GPS DATA \tr\n" );
			GpsPower( ON );				// maybe the power to the GPS module has been switched off.
			starttimer( GPS_TIMEOUT  );
		}

		if( get_nmea_sentence( gps_buff , GPS_BUF_LEN ) == OK  )
		{
			handle_mnea_sentence( gps_buff );
			starttimer( GPS_TIMEOUT );
		}

		idle();		// Reduces power consumption
	}
}
