//
// input.c
//
// implementation of the input functions
//
// Martin Jansen
//
// KeyboardInput function is based on the get_string() function from
// Oscar Jacobse
//
// For questions:
// email:	support@opticon.com
// web:		www.opticon.com
//
//
// 03/06/2005: 	Bug fix in display_input() function, cursor position was
//				not correctly calculated.
//
// 08/06/2005: 	Added function ScanBarcodeSymbol(), ScanOrKeyboardInputSymbol(),
//				Added INPUT_PRINT define (all printable characters)
//				Added Automatic cursor on when selecting Keyboard input
//				Automatic setecho(ON) and setecho(OFF) keyboardinput with INPUT_ALPHA or NPUT_PRINT
//				Changed display_input function to indicate difference between full string and max display_length
//
// 20/06/2005: 	Added KeyboardNumeric function for input a whole numeric values displaying is done from
//				left to right to give numeric input a more natural feeling
//
// 12/03/2007: 	Added KeyboardInput_SMS function for input of text simular to typing an SMS message on a mobile phone.
//              (added by Ronny de Winter)
//

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h> // needed for va_xxxx
#include <ctype.h>
#include <string.h>

#include "lib.h"
#include "input.h"

// Key sets for each key for the (SMS style) input function: KeyboardInput_SMS
#define MAX_KEYSET		9

const unsigned char keyset[11][MAX_KEYSET] =
{
	{' ', '_', '+', '-', '=', '!', '?', '0' ,'\0'},		// 0
	{' ', '.', ',', ':', ';', '@', '#', '1' ,'\0'},		// 1
	{'A', 'B', 'C', '2', '\0','\0','\0','\0','\0'},		// 2
	{'D', 'E', 'F', '3', '\0','\0','\0','\0','\0'},		// 3
	{'G', 'H', 'I', '4', '\0','\0','\0','\0','\0'},		// 4
	{'J', 'K', 'L', '5', '\0','\0','\0','\0','\0'},		// 5
	{'M', 'N', 'O', '6', '\0','\0','\0','\0','\0'},		// 6
	{'P', 'Q', 'R', 'S', '7', '\0','\0','\0','\0'},		// 7
	{'T', 'U', 'V', '8', '\0','\0','\0','\0','\0'},		// 8
	{'W', 'X', 'Y', 'Z', '9', '\0','\0','\0','\0'},		// 9
	{'*', '&', '%', '/', '\\','(' ,')' ,'\'', '.'},		// '.' or '*'
};

static void keybeep(int key)
{
#ifdef IS_BREAK_CODE
	if(IS_BREAK_CODE(key))		// Only beep if key is pressed
		return;
#endif

	sound( TSHORT, VMEDIUM, SHIGH, 0);
}

static void okbeep( void )
{
	sound( TSTANDARD, VMEDIUM, SHIGH, SMEDIUM, SHIGH, 0);
}

//
// Wait for keyboard input, when key is pressed
// return the key.
//
int WaitForKey( void )
{
	int c;

	while( ( c = getchar()) == EOF)
		idle(); // idle for powersaving
	
	keybeep(c);  // make the beeping sound
	return c;
}

//
// Check if the pressed key is found in the list
// made static so it won't show up in the map file
static int check_key_input( int key, int num, va_list key_list )
{
	int count;

	for( count = 0; count < num; count++ )
	{
		if( (int) va_arg( key_list, int ) == key )
		{
			va_end( key_list ); // reset the argument ptr
			return( OK );
		}
	}
	va_end( key_list );
	return( ERROR );
}

//
// Wait for one of the specific keys in the list
// If key is found in the list return the key
//
int WaitForKeys( int num, ... )
{
	va_list key_list;
	int key;

	if( num <= 0 )
		return( EOF ); // No keys in the list
	va_start( key_list, num );
	resetkey();
	for(;;)
	{
		key = WaitForKey(); // Wait for keyboard input
		if( check_key_input( key, num, key_list ) == OK )
		{
			va_end( key_list );	// reset the argument ptr
			return( key );
		}
	}
}

int ScanBarcodeSymbol( char* string, int min_length, int max_length, int *nCodeId)
{
	int key;
	struct barcode code = {0};

	(*nCodeId) = 0;
	resetkey();
	code.min = min_length;
	code.max = max_length;
	code.text = string;
	scannerpower( SINGLE, 300 );

	while( readbarcode( &code ) != OK )
	{
		if( kbhit() )
		{
			if(( key = getchar() ) != TRIGGER_KEY )
			{
				// put the character back in the input buffer
				// so it can be handled by another function
				ungetc( key, stdin);
				scannerpower( OFF, 0);
				return( ERROR );
			}
			scannerpower( SINGLE, 300 );

		}
		idle();
	}
    // Copy the code ID
    (*nCodeId) = code.id;
	goodreadled( GREEN, 10 );
	scannerpower( OFF, 0);
	okbeep();

	return( OK );
}

int ScanBarcode( char* string, int min_length, int max_length)
{
	int nCodeId = 0;

	return ScanBarcodeSymbol( string, min_length, max_length, &nCodeId);
}

static void remove_key_from_buffer( char* string )
{
	if( strlen( string ) > 0 )
		string[ strlen( string ) - 1 ] = '\0';
}


static void display_input( char* string, int x, int y, int display_length, int max_length )
{
	int length;
	int output;

	if( x < 0 || y < 0 )
		return;
	length = strlen( string );
	gotoxy( x, y);
	if( length < max_length )
		printf("%-*.*s ", display_length-1, display_length-1, string + ((length<display_length)?0:(length - display_length + 1)));
	else
		printf("%-*.*s", display_length, display_length, string + ( length - display_length ));

	output = (length < display_length)?(length+1):display_length;
	gotoxy( x + output -1, y ); // set cursor to the correct position
}

static void store_key_in_string( int key, char* string, int max_len, int typ )
{
	int length, ok;

	length = strlen( string );
	if( length >= max_len )
		return;

	ok = 0;

	if( typ & INPUT_FLOAT )
		ok = ( isdigit( key ) || ((strchr( string, '.') == NULL) && ( key == '.' )));
	if( !ok && ( typ & INPUT_NUM ))
		ok = isdigit( key );
	if( !ok && ( typ & INPUT_ALPHA ))
		ok = ( isalpha(	key ) || ( key == ' ' ));
	if( !ok && ( typ & INPUT_NEGATIVE ) && ( length == 0 ))
		ok = ( key == '-' );
	if( !ok && ( typ & INPUT_PRINT ))
		ok = isprint( key );

	if( ok )
	{
		string[ length++ ] = (char)key;
		string[ length ] ='\0';
	}
}

int KeyboardInput( char* string, int min_length, int max_length, int typ, int x, int y, int display_length, int num, ... )
{
	va_list key_list;
	int key;

	if( num <= 0 )
		return( EOF );
#if (PHL|PHL1000|PHL2700)
	if( typ & INPUT_ALPHA || typ & INPUT_PRINT)
		setecho( ON );
#endif
	if( display_length > max_length )
		display_length = max_length;
	display_input(string, x, y, display_length, max_length );

	cursor( ON );
	for(;;)
	{
		va_start( key_list, num );

		if( ((key = WaitForKey()) == ENT_KEY ) && (strlen( string ) < min_length ))
			continue;

		if( check_key_input( key, num, key_list ) == OK )
		{
			va_end( key_list );
#if (PHL|PHL1000|PHL2700)
			if( typ & INPUT_ALPHA || typ & INPUT_PRINT)
				setecho( OFF );
#endif
			cursor( OFF );
			return( key );
		}
		va_end( key_list );

		if( key == BS_KEY )
		{
			remove_key_from_buffer( string );
			display_input( string, x, y, display_length, max_length );
			continue;
		}
		store_key_in_string( key,  string, max_length, typ );
		display_input(string, x, y, display_length, max_length );
	}
}



int ScanOrKeyboardInput( char* string, int min_length, int max_length, int typ, int x, int y, int display_length )
{
	int	key;

	for(;;)
	{
		string[0] = '\0';
		display_input( string, x, y, display_length, max_length );
		if( ScanBarcode( string, min_length, max_length ) == OK )
		{
			display_input( string, x, y, display_length, max_length );
			return( SCANNED ); // Input by scanning
		}

		key = KeyboardInput( string, min_length, max_length, typ, x, y, display_length, 3, TRIGGER_KEY, CLR_KEY, ENT_KEY );
		switch( key )
		{
			case ENT_KEY:
				return( KEYBOARD );	// Keyboard input
			case TRIGGER_KEY:
				return( KEYBOARD );	// Keyboard input

			case CLR_KEY:
				if( strlen( string ) != 0)
				{
					break;
				} // fall through
			default:
				return( key );
		}
	}
}


int ScanOrKeyboardInputSymbol( char* string, int min_length, int max_length, int typ, int x, int y, int display_length, int *nCodeId )
{
	int	key;

	for(;;)
	{
		if( ScanBarcodeSymbol( string, min_length, max_length, nCodeId ) == OK )
		{
			display_input( string, x, y, display_length, max_length );
			return( SCANNED ); // Input by scanning
		}

		string[0] = '\0';
		display_input( string, x, y, display_length, max_length );

		key = KeyboardInput( string, min_length, max_length, typ, x, y, display_length, 4, TRIGGER_KEY, CLR_KEY, ENT_KEY, Q1_KEY );
		switch( key )
		{
			case ENT_KEY:
				return( KEYBOARD );	// Keyboard input
			case TRIGGER_KEY:
				return( KEYBOARD );	// Keyboard input

			case CLR_KEY:
				if( strlen( string ) != 0)
				{
					break;
				} // fall through
			default:
				return( key );
		}
	}
}

// ----------------------------------------------------------------------------
// Special numeric input below
// ----------------------------------------------------------------------------

static void store_char_in_string_numeric( int key, int max_len, char *string, int typ, int defaul )
{
	int i, ok;
	static char tmp[INPUT_BCR_MAX+1];

	max_len += ((typ & INPUT_NEGATIVE)&&(string[0]=='-'))?1:0;
	i = strlen( string );

	if(( i >= max_len ) && (key != '-') && !defaul)
		return;

	ok = 0;

	if( typ & INPUT_NUM )
		ok = isdigit( key );
	if( !ok && ( typ & INPUT_NEGATIVE ))
		ok = ( key == '-');
	if( ok )
	{
		if( defaul )
		{
			string[0] = (char)key;
			string[1] = '\0';
		}
		else
		{
			if( key == '-')
			{
				if( string[0] == '-' )
					strcpy( tmp, string+1 );
				else
				{
					strcpy( tmp, "-" );
					strcat(tmp, string );
				}
				strcpy( string, tmp );
			}
			else
			{
				string[ i++ ] = (char)key;
				string[i] = '\0';
			}
		}
	}
}

static void display_input_numeric( char* string, int x, int y, int max_x )
{
	int length;

	if( x < 0 || y < 0 )
		return;

	length = strlen( string );

	gotoxy(x,y);
	if( length > max_x )
	{
		length -= max_x;
		printf("%*.*s", max_x, max_x, string+length );
	}
	else
		printf("%*.*s", max_x, max_x, string );

	gotoxy( x+max_x-1, y); // set the cursor to the correct position
}

//
// Special input function for numeric input
// Alignment is done from right to left
//
// max_length is the maximum length without the minus sign (-) if type also has INPUT_NEGATIVE
// then max length is increase by 1
//
int KeyboardNumeric( char* string, int max_length, int typ, int x, int y, int display_length, int num, ... )
{
	va_list key_list;
	int key;
	int defaul;

	if( num <= 0 )
		return( EOF );

	if( typ & ~(INPUT_NEGATIVE|INPUT_NUM|INPUT_SHOW_DEFAULT))
		return( EOF ); // Only INPUT_NEGATIVE and INPUT_NUM are allowed

	if( typ & INPUT_SHOW_DEFAULT )
		defaul = TRUE;
	else
	{
		string[0] = '\0';
		defaul = FALSE;
	}
	cursor( ON );
	for(;;)
	{
		va_start( key_list, num );

		display_input_numeric(string, x, y, display_length );

		key = WaitForKey();

		if( check_key_input( key, num, key_list ) == OK )
		{
			va_end( key_list );

			cursor( OFF );
			return( key  );
		}
		va_end(key_list);

		if( key == BS_KEY )
		{
			if( defaul )
				string[0] = '\0';
			remove_key_from_buffer( string );
			defaul = FALSE;	// default value is turned off
			continue;
		}

		store_char_in_string_numeric( key,  max_length, string, typ, defaul );
		defaul = FALSE;	// default value is turned off
	}
}





// Local variables for KeyboardInput_SMS
static unsigned char last_key;		// Contains previously pressed key
static short keyset_pos;			// Contains the selected position of the key set
static short shift_state;			// Contains the state of the shift key (Lower case / Shift 1 / Caps lock / Numeric mode)
static short key_timer_running;		// True if key time-out is active
static short key_time_out;			// Contains the key entry time-out
static short mark_last;				// True is last character of the string is marked/selected

// Local functions for KeyboardInput_SMS
static void display_input_SMS( char* string, int x, int y, int display_length, int max_length, int mark_last );
static void display_shift_state(void);
static void next_shift_state(void);
static void store_key_in_string_SMS( int key, char* string, int max_len );
static unsigned char get_char_from_keyset( int key, int keyset_pos);
static void display_keyset( int key, int keyset_pos);
static int is_input_key(int key);
static int WaitForKey_TO( void );
//

//-----------------------------------------------------------------------------
// Purpose:     Input a text string by the keyboard (SMS style)
//
// Parameters:  string		- holds the typed data string
//
//              min_length	- the minimal length the data at least needs to be
//
//				max_length	- the maximum length the data may be
//
//				x			- x position on the display
//
//				y			- y position on the display
//
//				display_length - the amount of characters to display before starting to scroll data
//
//				shift_init  - Initial state of the shift key when entering the function (Lower case / Shift 1 / Caps lock / Numeric mode)
//
//              key_timeout - Key entry time-out in steps of 20ms. When the time-out passes, the cursor moves to the next position. (0 is no time-out)
//
// Remarks 		CLR_KEY (when string is empty) and ENT_KEY (when string length >= min_length) are the exception key to exit the function
//
// Returns:     The exception key that exits the function is returned
//
int KeyboardInput_SMS( char* string, int min_length, int max_length, int x, int y, int display_length, int shift_init, int key_timeout)
{
	int length, key;
#ifdef IS_BREAK_CODE	
	int pressed;		// True as long as key isn't released

	pressed = FALSE;
#endif

	if( display_length > max_length )
		display_length = max_length;

	length = strlen(string);
	last_key = '\0';
	shift_state = shift_init;
	mark_last = FALSE;

	key_timer_running = FALSE;
	key_time_out = key_timeout;

	if(key_time_out > 0 )
		starttimer(key_time_out);
	else
		starttimer(0);

	display_shift_state();
	display_input_SMS(string, x, y, display_length, max_length, mark_last );

	cursor( ON );

	systemsetting("*R");	// "Handle Shift key by Application"
	systemsetting("[D3N");	// "Enable receiving of keyboard break codes" (If not supported, this call won't change anything

	for(;;)
	{
		if( ((key = WaitForKey_TO()) == ENT_KEY ) && (strlen( string ) < min_length ))
			continue;

		// Key entry time out, so currently selected character is entered
		if(key == EOF && mark_last)
		{
#ifdef IS_BREAK_CODE
			if(pressed)		// If time-out occurs with the current key still pressed -> set key position to numeric value
			{
				keyset_pos = MAX_KEYSET - 1;

				while( get_char_from_keyset(last_key, keyset_pos) == '\0' )	// Set key position to numeric value
					--keyset_pos;

				remove_key_from_buffer( string );							// Overwrite current key with numeric value
				store_key_in_string_SMS( last_key,  string, max_length );
			}
#endif
			keybeep(key);  // make the beeping sound
			key = Q2_KEY;
		}

#ifdef IS_BREAK_CODE
		if(is_input_key(key) && !IS_BREAK_CODE(key) )
			pressed = TRUE;
		else
			pressed = FALSE;
#endif

		keybeep(key);  // make the beeping sound

		switch (key)
		{
			case CLR_KEY:
				if(strlen(string) != 0)
				{
					if(last_key == CLR_KEY && !endtimer())
					{
						string[0] = '\0';
						last_key = (unsigned char)key;
						mark_last = FALSE;
						break;
					}

					if(mark_last)
					{
						remove_key_from_buffer( string );
						mark_last = FALSE;
					}

					starttimer(25);
					last_key = (unsigned char)key;
					break;
				}
				// Fall through
			case ENT_KEY:
				cursor( OFF );
				systemsetting("*Q");	// "Handle Shift key by OS",
				systemsetting("[D3O");	// "Disable receiving of keyboard break codes"
				return( key );

			case Q2_KEY:
				if(shift_state == CAPS_ONCE && mark_last && is_input_key(last_key) )
				{
					if(shift_state == CAPS_ONCE)
						shift_state = CAPS_OFF;		// Release shift key
				}

				last_key = (unsigned char)key;
				mark_last = FALSE;
				break;

			case BS_KEY:
				remove_key_from_buffer( string );
				last_key = (unsigned char)key;
				mark_last = FALSE;
				break;

			case SHIFT_KEY:

				next_shift_state();	// Set next shift state

				if(mark_last && is_input_key(last_key))
				{
					remove_key_from_buffer( string );
					store_key_in_string_SMS( last_key,  string, max_length );
				}

				break;

			//* Scroll back in key list
			case Q1_KEY:
			case TRIGGER_KEY:
				if(mark_last == FALSE)
				{
					last_key = (unsigned char)key;
					break;
				}

				if(key == Q1_KEY)
					key = '-';
				else
					key = '+';
				// Fall Through
			default:

				if( is_input_key(key) || key == '+' || key == '-')
				{
					if( shift_state == NUM_LOCK )
					{
						if(mark_last)		// If a character is still selected overwrite that character
							remove_key_from_buffer( string );

						last_key = (unsigned char)key;
						store_key_in_string_SMS( key,  string, max_length );
						mark_last = FALSE;
					}
					else
					{
						if(key == last_key || key == '+')
						{
							key = (unsigned char)last_key;

							++keyset_pos;

							if( get_char_from_keyset(key, keyset_pos) == '\0' )
								keyset_pos = 0;
							else if(keyset_pos >= MAX_KEYSET)
								keyset_pos = 0;

							remove_key_from_buffer( string );
						}
						else if(key == '-')
						{
							key = (unsigned char)last_key;

							if(--keyset_pos < 0)
								keyset_pos = MAX_KEYSET - 1;

							while( get_char_from_keyset(key, keyset_pos) == '\0' )
								--keyset_pos;

							remove_key_from_buffer( string );
						}

						else
							keyset_pos = 0;

						if(key != (unsigned char)last_key)
						{
							// Release shift (except on first
							if(shift_state == CAPS_ONCE && is_input_key(last_key))
								shift_state = CAPS_OFF;

							last_key = (unsigned char)key;
						}

						store_key_in_string_SMS( key,  string, max_length );

						mark_last = TRUE;
					}
				}
				break;
		}

		// Display the shift state  (Lowercase / shift / caps lock / num lock )
		display_shift_state();

		// Display the key set of the currently selected character
		display_keyset( ((mark_last == FALSE) ? 0 : last_key), keyset_pos);

		// Display the input string
		display_input_SMS(string, x, y, display_length, max_length, mark_last );
	}
}

// Wait for keyboard input, when key is pressed return the key.
int WaitForKey_TO( void )
{
	int c;

	if(!mark_last)
		key_timer_running = FALSE;

	while( (c = getchar()) == EOF && !(key_timer_running && endtimer() && mark_last) )
	{
		idle(); // idle for powersaving
	}

	if( c == EOF )
	{
		key_timer_running = FALSE;
	}
	else if( key_time_out > 0 && c != CLR_KEY)
	{
		starttimer(key_time_out);
		key_timer_running = TRUE;
	}

	keybeep(' ');  // make the beeping sound

	return c;
}

// Store key at last position in string
static void store_key_in_string_SMS( int key, char* string, int max_len )
{
	int length;
	unsigned char ch;

	length = strlen( string );

	if(max_len == 0)
		return;

	if( length >= max_len )						// If length >= max_len
		length = max_len - 1;					// Overwrite last character of string

	if(shift_state != NUM_LOCK)
	{
		ch = get_char_from_keyset(key, keyset_pos);		// Get key from keyset table

		if(shift_state == CAPS_OFF && ch >= 'A' && ch <= 'Z')		// If shift mode = lower case
			ch += 0x20;		// Convert to lower case
	}
	else
		ch = (unsigned char)key;	// Numeric -> Store key as is

	string[ length++ ] = (char)ch;
	string[ length ] ='\0';
}

// Display input string_SMS
static void display_input_SMS( char* string, int x, int y, int display_length, int max_length, int mark_last )
{
	int length;
	int output;
	int xpos, ypos;
	unsigned int height=0, width=0;

	if( x < 0 || y < 0 )
		return;

	getfontsize( &height, &width);
    width = DISP_WIDTH / width;

	length = strlen( string );
	gotoxy( x, y);

	if( length < max_length )
		printf("%-*.*s ", display_length-1, display_length-1, string + ((length<display_length)?0:(length - display_length + 1)));
	else
		printf("%-*.*s", display_length, display_length, string + ( length - display_length ));

	output = (length < display_length)?length:(display_length-1);

	if(mark_last && length != 0 && length != max_length)	// if mark_last is set, select (invert) the last character
		--output;

	xpos = (x+output) % width;
	ypos = (x+output) / width + y;

	if(mark_last && length != 0)	// if mark_last is set, select (invert) the last character
	{
		gotoxy( xpos, ypos ); // set cursor to the correct position
		printf("\tr%c\tr", string[length - 1]);
	}

	gotoxy( xpos, ypos ); // set cursor to the correct position
}

// Print shift state
static void display_shift_state(void)
{
	int x, y;
	unsigned int height=0, width=0;

	getfontsize( &height, &width);

	height = DISP_HEIGHT/height;
	width = DISP_WIDTH/width;

	x = (SHIFT_STATE_POSITION & ALIGN_RIGHT) ? (width - 3) : 0;
	y = (SHIFT_STATE_POSITION & ALIGN_BOTTOM) ? (height - 1) : 0;

	gotoxy(x, y);

	switch(shift_state)
	{
		case CAPS_ONCE:
			printf("\trAbc\tr");
			break;
		case CAPS_LOCK:
			printf("\trABC\tr");
			break;
		case NUM_LOCK:
			printf("\tr123\tr");
			break;
		case CAPS_OFF:
		default:
			printf("\trabc\tr");
			break;
	}
}

// Next shift state
static void next_shift_state(void)
{
	switch(shift_state)
	{
		case CAPS_OFF:
			shift_state = CAPS_ONCE;
			break;
		case CAPS_ONCE:
			shift_state = CAPS_LOCK;
			break;
		case CAPS_LOCK:
			shift_state = NUM_LOCK;
			break;
		case NUM_LOCK:
			shift_state = CAPS_OFF;
			break;
	}
}

//Select character from keyset
static unsigned char get_char_from_keyset( int key, int keyset_pos)
{
	if( key >= '0' && key <= '9')
		return keyset[key-'0'][keyset_pos];

	if (key == '*' || key == '.')
		return keyset[10][keyset_pos];

	return '\0';
}

// Display the keyset of the pressed key at a given position and alignment (ALIGN_LEFT or ALIGN_RIGHT)
static void display_keyset( int key, int keyset_pos)
{
	int i, xpos;
	int align_right, align_bottom;
	int x, y;
	unsigned int height=0, width=0;
	unsigned char ch;
	
	getfontsize( &height, &width);

	height = DISP_HEIGHT/height;
	width = DISP_WIDTH/width;

	align_right  = (KEYSET_POSITION & ALIGN_RIGHT)  ? TRUE : FALSE;
	align_bottom = (KEYSET_POSITION & ALIGN_BOTTOM) ? TRUE : FALSE;

	x = (align_right) ? (width - 1) : 0;
	y = (align_bottom) ? (height - 1) : 0;

	// If any key without a key set or in numeric mode, just clear the line
	if( !is_input_key(key) || shift_state == NUM_LOCK)
	{
		gotoxy(x,y);

		for(i=0;i<MAX_KEYSET;++i)
			putchar(' ');

		return;
	}

	i = (align_right) ? MAX_KEYSET - 1: 0;

	xpos = x;

	while( (!align_right && i < MAX_KEYSET) || ((align_right != 0) && i >= 0) )
	{
		ch = get_char_from_keyset(key, i);

		if(shift_state == CAPS_OFF && ch >= 'A' && ch <= 'Z')
			ch += 0x20;					// Convert to lower case

		gotoxy(xpos,y);

		if( ch != '\0')
		{
			if(i == keyset_pos)
				printf("\tr%c\tr", ch);		// Mark the currently selected character
			else
				putchar(ch);

			xpos += (align_right) ? (-1) : (1);
		}

		i += (align_right) ? (-1) : (1);
	}

	
	if( align_right != 0 )
	{
		for(;xpos >= x - MAX_KEYSET; --xpos)
		{
			gotoxy(xpos,y);
			putchar(' ');
		}
	}
	else
	{
		for(;xpos < x + MAX_KEYSET; ++xpos)
		{
			gotoxy(xpos,y);
			putchar(' ');
		}
	}
}

//  Returns true if a numeric, '*' or '.' is pressed
static int is_input_key(int key)
{
	return ((key >= '0' && key <= '9') || (key == '*' || key == '.'));
}


