//
// input.h
//
// header file of some input functions for the
// opticon barcode terminals
//
// Martin Jansen
//
// KeyboardInput function is based on the get_string() function from
// Oscar Jacobse
//
// For questions:
// email:	support@opticon.com
// web:		www.opticon.com
//
// 03/06/2005: 	Bug fix in display_input() function, cursor position was
//				not correctly calculated.
//
// 08/06/2005: 	Added function ScanBarcodeSymbol(), ScanOrKeyboardInputSymbol(),
//				Added INPUT_PRINT define (all printable characters)
//				Added Automatic cursor on when selecting Keyboard input
//				Automatic setecho(ON) and setecho(OFF) keyboardinput with INPUT_ALPHA or NPUT_PRINT
//				Changed display_input function to indicate difference between full string and max display_length
//
// 20/06/2005: 	Added KeyboardNumeric function for input a whole numeric values displaying is done from
//				left to right to give numeric input a more natural feeling
//
//

#ifndef __INPUT_H__
#define __INPUT_H__

#ifndef DISP_WIDTH
#if PHL        // PHL1700-10 and PHL1700-20
   #define DISP_WIDTH  96
   #define DISP_HEIGHT 48
#elif PHL1000  // PHL1300
   #define DISP_WIDTH  96
   #define DISP_HEIGHT 64 // Change height to 48 when indicators need to be on (see cursor() function)
#elif PHL2700  // PHL2700
   #define DISP_WIDTH  128
   #define DISP_HEIGHT 64
#elif OPH	  // OPH1000
   #define DISP_WIDTH  112
   #define DISP_HEIGHT 128
#else          // OPL97XX and DCL153x series
   #define DISP_WIDTH  112
   #define DISP_HEIGHT 64
#endif
#endif // DISP_WIDTH



#define INPUT_FLOAT		0x01
#define INPUT_NUM		0x02
#define INPUT_ALPHA		0x04
#define INPUT_NEGATIVE	0x08
#define INPUT_PRINT		0x10
#define INPUT_ALL		(INPUT_FLOAT | INPUT_NUM | INPUT_ALPHA | INPUT_NEGATIVE | INPUT_PRINT )

#define INPUT_SHOW_DEFAULT	0x80

#define INPUT_BCR_MAX	50

#define DEFAULT			0x200 	// default return value
#define KEYBOARD		0x400	// used when input is done by keyboard
#define SCANNED			0x800	// used when input is scanned

// Defines for KeyboardInput_SMS
#define CAPS_OFF		0		// Lower case
#define CAPS_ONCE		1		// Shift 1
#define CAPS_LOCK		2		// Caps lock
#define NUM_LOCK		3		// Numeric mode

#define ALIGN_LEFT		0		// Align left
#define ALIGN_TOP		0		// Align top

#define ALIGN_RIGHT		1		// Align right
#define ALIGN_CENTER	2		// Align centered
#define ALIGN_BOTTOM	4		// Align top

#define KEYSET_POSITION			(ALIGN_LEFT|ALIGN_BOTTOM)		// Change position to your own desire

#define SHIFT_STATE_POSITION	(ALIGN_RIGHT|ALIGN_BOTTOM)		// Change position to your own desire

//-----------------------------------------------------------------------------
// Purpose:     Wait until any key is pressed
//
// Parameters:  None
//
// Returns:     The key that is pressed
//
int WaitForKey( void );

//-----------------------------------------------------------------------------
// Purpose:     Wait until one of the keys from the list is pressed
//
// Parameters:  num			- amount of keys in the list
//
//              ...			- The list of the keys to react on
//
// Returns:     The key that was pressed from the list
//
int WaitForKeys( int num, ... );

//-----------------------------------------------------------------------------
// Purpose:     Scan a barcode
//
// Parameters:  string		- holds the scanned barcode
//
//              min_length	- the minimal length the barcode at least needs to be
//
//				max_length	- the maximum length the barcode may be
//
// Returns:     OK on success, ERROR when any other key is pressed
//
int ScanBarcode( char* string, int min_length, int max_length);

//-----------------------------------------------------------------------------
// Purpose:     Scan a barcode
//
// Parameters:  string		- holds the scanned barcode
//
//              min_length	- the minimal length the barcode at least needs to be
//
//				max_length	- the maximum length the barcode may be
//
//				nCodeId		- returns the matching code id of the barcode (see lib.h for matchin code ID
//
// Returns:     OK on success, ERROR when any other key is pressed
//
int ScanBarcodeSymbol( char* string, int min_length, int max_length, int *nCodeId);

//-----------------------------------------------------------------------------
// Purpose:     Input a string of data by the keyboard
//
// Parameters:  string		- holds the typed data string
//
//              min_length	- the minimal length the data at least needs to be
//
//				max_length	- the maximum length the data may be
//
//				typ			- orred parameter for FLOAT, NUMERIC, ALPHA, NEGATIVE
//
//				x			- x position on the display
//
//				y			- y position on the display
//
//				display_length - the amount of characters to display before starting to scroll data
//
//				num			- amount of exception keys
//
//				...			- list of exception keys
//
// Remarks 		BS_KEY cannot be used as an exception key
//
// Returns:     EOF when no exception keys are used, else returns the exception key
//
int KeyboardInput( char* string, int min_length, int max_length, int typ, int x, int y, int display_length, int num, ... );

//-----------------------------------------------------------------------------
// Purpose:     Input a text string by the keyboard (SMS style)
//
// Parameters:  string		- holds the typed data string
//
//              min_length	- the minimal length the data at least needs to be
//
//				max_length	- the maximum length the data may be
//
//				x			- x position on the display
//
//				y			- y position on the display
//
//				display_length - the amount of characters to display before starting to scroll data
//
//				shift_init  - Initial state of the shift key when entering the function (Lower case / Shift 1 / Caps lock / Numeric mode)
//
//              key_timeout - Key entry time-out in steps of 20ms. When the time-out passes, the cursor moves to the next position. (0 is no time-out)
//
// Remarks: 	- The Backspace-key removes 1 character from the string. If the CLR-key is double-pressed quickly, the whole string is cleared.
//              - Use the SHIFT-key to switch between: Lower case (=abc), Shift (=Abc), Caps lock (=ABC) and Numeric mode (=123)
//              - To change the position of the displayed key-set to a different corner, change the global define: KEYSET_POSITION, for instance to (ALIGN_LEFT|ALIGN_TOP) or (ALIGN_RIGHT|ALIGN_BOTTOM)
//              - To change the position of the displayed shift state, change the global define: SHIFT_STATE_POSITION, for instance to (ALIGN_RIGHT|ALIGN_TOP) or (ALIGN_LEFT|ALIGN_BOTTOM)
//				
//
// Returns:     The key that exits the function is returned
//					CLR_KEY (only possible when string is empty) 
//					ENT_KEY (possible when string length >= min_length)
//
int KeyboardInput_SMS( char* string, int min_length, int max_length, int x, int y, int display_length, int shift_init, int key_timeout);

//-----------------------------------------------------------------------------
// Purpose:     Input a string of data by the keyboard, or by a scanned barcode
//
// Parameters:  string		- holds the typed data string
//
//              min_length	- the minimal length the data at least needs to be
//
//				max_length	- the maximum length the data may be
//
//				typ			- orred parameter for FLOAT, NUMERIC, ALPHA, NEGATIVE
//
//				x			- x position on the display
//
//				y			- y position on the display
//
//				display_length - the amount of characters to display before starting to scroll data
//
// Returns:     SCANNED on scanned barcode, KEYBOARD on a typed string or CLR_KEY when CLR_KEY was pressed
//
int ScanOrKeyboardInput( char* string, int min_length, int max_length, int typ, int x, int y, int display_length );


//-----------------------------------------------------------------------------
// Purpose:     Input a string of data by the keyboard, or by a scanned barcode
//
// Parameters:  string		- holds the typed data string
//
//              min_length	- the minimal length the data at least needs to be
//
//				max_length	- the maximum length the data may be
//
//				typ			- orred parameter for FLOAT, NUMERIC, ALPHA, NEGATIVE
//
//				x			- x position on the display
//
//				y			- y position on the display
//
//				display_length - the amount of characters to display before starting to scroll data
//
//				nCodeId		- the scanned barcode code id (see lib.h for matching info)
//
// Returns:     SCANNED on scanned barcode, KEYBOARD on a typed string or CLR_KEY when CLR_KEY was pressed
//
int ScanOrKeyboardInputSymbol( char* string, int min_length, int max_length, int typ, int x, int y, int display_length, int *nCodeId );


//-----------------------------------------------------------------------------
// Purpose:     Input a numeric string of data by the keyboard negative values allowed
//
// Parameters:  string		- holds the typed data string
//
//				max_length	- maximum number of digits without the minus sign (-) if type
//							  also has INPUT_NEGATIVE then max length is increase by 1
//
//				typ			- orred parameter for INPUT_NUMERIC, INPUT_NEGATIVE, INPUT_SHOW_DEFAULT
//
//				x			- x position on the display
//
//				y			- y position on the display
//
//				display_length - the amount of characters to display before starting to scroll data
//
//				num			- amount of exception keys
//
//				...			- list of exception keys
//
// Remarks 		BS_KEY cannot be used as an exception key
//
//
// Returns:     EOF when no exception keys are used, else returns the exception key
//
int KeyboardNumeric( char* string, int max_length, int typ, int x, int y, int display_length, int num, ... );

#endif // __INPUT_H__



