//------------------------------------------------------------------
//  Demo application for the OPN2002 and OPN2003
//------------------------------------------------------------------
//  This simple demo allows the OPN2002/3 to read barcodes and 
//  transmit them using the mini USB cable to a virtual comport 
//  on a PC. 
//
//  This application also support the reading of the 
//  univeral menu-book, so all barcode reading capabilities can 
//  easily been tested. Since all changed configurations are
//  stored in flash, the changed configurations are restored 
//  after a restart even if the battery has been empty.
//
//  This application does not demonstrate the storing of barcode 
//  data in a file or in flash memory
//------------------------------------------------------------------
//  Version history:
//     RFF35100		:	Release data: August 6th 2009
//						Author: Ronny de Winter
//						Company: Opticon Sensors Europe BV
//------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include "lib.h"

#define	MAX_DEMO_BARCODE_LENGTH		300

static char bcr_buf[MAX_DEMO_BARCODE_LENGTH];
static struct barcode code = {0};

//------------------------------------------------------------------
//  Demo_ProcessBarcode
//  =====================
//  Process a barcode that has been read. This function
//  makes is possible to read set-up labels from the universal 
//  book as well. (Changed settings will be lost after a restart)
//------------------------------------------------------------------
int Demo_ProcessBarcode(struct barcode *code)
{
	int result = 0;
	
	if(code->id == MENU_CODE)
	{
		scannerpower(ON, -1);

		while(result != EXITING_MENU_MODE && result != ERROR)
		{
			if(result == 0 || readbarcode(code) == OK)
			{
				switch( (result=ExecuteMenuLabel(code)) )
				{
					case ENTERING_MENU_MODE:
						sound(TSTANDARD, VHIGH, SHIGH, SMEDIUM, SHIGH, 0);
						break;

					case EXITING_MENU_MODE:
						scannerpower(OFF, 0);
						sound(TSTANDARD,VHIGH, SHIGH, SMEDIUM, SHIGH, 0);
						delay(TSTANDARD * 4);		// Wait till buzzer sound ends before saving (saving flash settings temp. turns of sound timer)
						systemsetting("Z2");		// Save changed settings in flash memory
						break;

					case INVALID_OPTION_READ:
						sound(TLONG, VHIGH, SLOW, 0);
						break;

					case VALID_OPTION_READ:
						sound(TSTANDARD, VHIGH, SHIGH, SMEDIUM, SHIGH,0);
						break;

					case LABEL_IGNORED:
						break;

					case ERROR:
						scannerpower(OFF, 0);
						break;		
				}
			}
		}

		return TRUE;
	}
	else if(code->id == MENU_CODE_PDF || code->id == MENU_CODE_C128)
	{
		switch( (result=ExecuteMenuLabel(code)) )
		{
			case INVALID_OPTION_READ:
				scannerpower(OFF, 0);
				sound(TLONG, VHIGH, SLOW, 0);
				break;

			case VALID_OPTION_READ:
				scannerpower(OFF, 0);
				sound(TSTANDARD, VHIGH, SHIGH, SMEDIUM, SHIGH, SMEDIUM, SHIGH, SMEDIUM, SHIGH,0);
				delay(TSTANDARD * 8);		// Wait till buzzer sound ends before saving (saving flash settings temp. turns of sound timer)
				systemsetting("Z2");		// Save changed settings in flash memory
				break;

			case LABEL_IGNORED:
			default:
				break;		
		}

		return TRUE;
	}

	goodreadled(GREEN, 50);
	sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, SMEDIUM, NULL);
	printf("%s\r", code->text);
	return FALSE;
}



//------------------------------------------------------------------
//  main
//  =====================
//  Main function of this demo that allows the OPN200x to read 
//  barcodes and transmit them using the mini USB cable to a virtual 
//  comport on a PC or laptop. This application also support the 
//  reading of the univeral menu-book. Since all changed 
//  configurations are stored in flash, the changed configurations 
//  are restored after a restart even if the battery has been empty.
//------------------------------------------------------------------
void main(void)
{
	comopen(COM0);

	code.text = bcr_buf;
	code.min = 0;
	code.max = MAX_DEMO_BARCODE_LENGTH;

	if(SystemSettingsMemorizing(ON) < 0)	// Enable remembering of (decoder) settings in flash and load old settings if present
	{
		systemsetting("U2");				// If no old settings were found -> Reset to default and save settings
	}

	systemsetting("YC");					// Always make sure menu-labels are enabled after a restart

	autopowerdown(ON, 3 * 50);				// 3 seconds power down time (when idle)

	scannerpower(TRIGGER | SINGLE, 250);	// Trigger mode, 5 seconds read time, laser off after 1 barcode 

	for(;;)
	{
		if(readbarcode(&code) == OK)
		{
			if(Demo_ProcessBarcode(&code) == TRUE)
			{
				while(triggerpressed())						// Wait for release before resetting read mode
					idle();

				scannerpower(TRIGGER | SINGLE, 250);		// Reset read mode, because it's turned off after reading menu labels
			}
		}

		idle();		// This reduces the power consumption
	}
}
