/* ######  ##   ## ####        #####   ####### ##   ##  #####             */
/*  ##  ## ##   ##  ##          ## ###  ##   # ### ### ##   ##            */
/*  ##  ## ##   ##  ##          ##  ##  ##     ####### ##   ##            */
/*  #####  #######  ##          ##  ##  ####   ## # ## ##   ##            */
/*  ##     ##   ##  ##          ##  ##  ##     ##   ## ##   ##            */
/*  ##     ##   ##  ##  ##      ## ###  ##   # ##   ## ##   ##            */
/* ###     ##   ## #######     #####   ####### ##   ##  #####             */
/*                                                                        */
/* Opticon PHL2700 RFID test program					                  */
/* Reads, writes and locks 13,56Mhz RFID Tags and tests the RFID module   */
/* Supported tags: Tag-it, I-Code, ISO-15693                              */
/* Required RFID firmware version HAAV0007 or higher                      */
/* Minimum OS version CBWV0126 (recommended CBWV0145 or higher)           */
/* Compiler: Microtec C compiler for Melps 7700  Version 2.2              */
/* Written by: R. de Winter                                               */

/************************************************************************/
/*      Include                                                         */
/************************************************************************/
#include		<stdio.h>
#include		<stdlib.h>
#include		<string.h>
#include        <ctype.h>
#include        "lib.h"
#include		"rfidlib.h"

/************************************************************************/
/*      Define                                                          */
/************************************************************************/
#define VERSIONSUFFIX  "20964"
#define NOT_OK         -1

#define LOCKED         -2

#define CLEAR          -1
#define SCAN            1
#define KEYS            2
#define ALPHA           8
#define NUM             16
#define FLOAT           32
#define NONE            1
#define EVEN            2
#define ODD             3

#define ICODE			0
#define	TAGIT			1
#define ISO15693		2
#define ISO15693_OMRON  3

struct xmenus {
	char *item;                 // menu item string
    void(* itemfunc)(void);     // function to be executed on selection of item
};

/************************************************************************/
/*      Global                                                          */
/************************************************************************/
static unsigned char write_data[8+1];

/************************************************************************/
/*      extern                                                          */
/************************************************************************/
/************************************************************************/
/*      Prototype                                                       */
/************************************************************************/
int xmenu(const char *menu_text, const struct xmenus *tab, const int max);
int get_string(char *string, int *code_id, int typ, int smin, int smax, int kmax, int x, int y);
void wait_key(void);

void main_xmenu(void);
void icode_xmenu(void);
void tagit_xmenu(void);
void iso15693_xmenu(void);
void test_xmenu(void);

//Interface functions of the read,write,lock and status functions
void read_main(void);
void write_main(void);
void read_lockstatus_main(void);
void lock_block_main(void);

//Actual implementation of the read,write,lock and status functions
int read_tag(int rfidtype, char *bufp);
int write_tag(int rfidtype, char* data);
int read_lockstatus(int rfidtype, unsigned char nr_of_blocks);
int lock_block(int rfidtype, unsigned char block_number);

//General Tag functions
int AutoTagTypeDetect(char *bufp);
int tagpower(int onoff);
int show_taginfo(int rfidtype, char *tagcode, unsigned char *write_data);

//I-Code only functions
void icode_reset_quiet_mode(void);
void icode_family_appl(void);

//Tag-it only functions
void tagit_put_lock(void);

//ISO-15693 only functions
void iso15693_AFI(void);
void iso15693_DSFID(void);
void iso15693_AFI_DSFID(int testAFI);

//Test and info functions
void show_versions(void);
void test_mode1(void);
void test_mode2(void);

/************************************************************************/
/* the execution function for an action menu                            */
/************************************************************************/
int xmenu(const char *menu_text, const struct xmenus *tab, const int max)
{
	int count;
	int index;
	int calcu;
	int key;
	index = 0;
	count = 0;
	putchar('\f');				/* clear screen */
	for (;;) {
		gotoxy(0, 0);
		if (index == 0)
			printf(" %s%c", menu_text, 31);
		else if (index == max)
			printf("%c%s ", 30, menu_text);
		else
			printf("%c%s%c", 30, menu_text, 31);
		if (index > 2)
			calcu = index - 2;
		else
			calcu = 0;
		for (count = 0; count < 3; count++) {
			gotoxy(0, count + 1);
			if (index == (count + calcu))
				printf("\tr%s\tr", tab[count + calcu].item);
			else
                if ((count+calcu)>max)
                    printf("              ");
                else
				    printf("%s", tab[count + calcu].item);
		}

		while ((key=getchar())==EOF)
		{   if (powereddown())
				backlight(OFF);
			idle();
		}

		switch (key) {

		case BL_KEY:
		case F4_KEY:
			backlight(TOGGLE);
			break;

		case UP_KEY:
		case LEFT_KEY:
		case F7_KEY:
		case F5_KEY:
		case Q1_KEY:
			if (index > 0)
				index--;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;


		case DOWN_KEY:
		case RIGHT_KEY:
		case F8_KEY:
		case F6_KEY:
		case Q2_KEY:
			if (index < max)
				index++;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;


		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			if ((key - '1') <= max)
				return (key - '1');
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;

		case ESC:
			return (-1);
		case ENT_KEY:
		case TRIGGER_KEY:
			return (index);

		default:
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		}
	}
}

int get_string(char *string, int *code_id, int typ, int smin, int smax, int kmax, int x, int y)
{
	static char buf[42];
	static char bcr_buf[42];
	struct barcode code;
	int result;
	int key;
	int count;
	int ok;

	strcpy(buf, string);
	cursor(NOWRAP);
	cursor(ON);
	setecho(ON);
	resetkey();
    *code_id = 0;
	if (typ & SCAN) {
		code.text = bcr_buf;
		code.min = smin;
		code.max = smax;
		scannerpower(SINGLE, 300);
	}
	count = 0;
	gotoxy(x, y);
	buf[0] = '\0';
	for (;;) {
		while (((key=getchar())==EOF) && scanneroff())
		{	if (powereddown())
				backlight(OFF);
			idle();
		}
		if ((count==0) && (typ&SCAN) && ((key==TRIGGER_KEY)||(!scanneroff())) ) {
			while ((result = readbarcode(&code)) != OK) {
				if (powereddown())
					backlight(OFF);
				if (scanneroff())
					scannerpower(SINGLE, 300);
				if (kbhit())
					if ((key = getchar()) != TRIGGER_KEY)
						break;
			}
			if (result == OK) {
				scannerpower(OFF, 0);
				goodreadled(GREEN, 10);
				sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, NULL);
				strcpy(buf, code.text);
				gotoxy(x, y);
				if (code.length >= (16 - x))
					printf("%s", buf + (code.length - (16 - x)));
				else
					printf("%s", buf);
				strcpy(string, buf);
				cursor(OFF);
				setecho(OFF);
				*code_id = code.id;
				return (SCAN);
			}
		}
		scannerpower(OFF, 0);
		sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
		if (!(typ & KEYS)) {
			if (key == ESC) {
				cursor(OFF);
				setecho(OFF);
				return (CLEAR);
			} else
				continue;
		}
		if ((key==BL_KEY)||(key==F4_KEY))
			backlight(TOGGLE);

		if (typ & KEYS) {
			switch (key) {
			case DOWN_KEY:
			case RIGHT_KEY:
			case UP_KEY:
			case LEFT_KEY:
			case F5_KEY:
			case F6_KEY:
			case F7_KEY:
			case F8_KEY:
				break;
			case TRIGGER_KEY:
				if (count > 0) {
					memset(buf, ' ', count);
					buf[count] = '\0';
					gotoxy(x, y);
					printf("%s", buf);
					gotoxy(x, y);
				}
				count = 0;
				buf[0]='\0';
				scannerpower(SINGLE, 300);
				break;

			case '\b':			/* backspace */

				if(typ != (NUM | KEYS))
				{
					if (count > 0)
					{
						buf[--count] = '\0';
						if ((x + count) > 13)
						{
							gotoxy(x, y);
							printf("%s", buf + (count - 14 + x));
						}
						else
						{
							gotoxy(x + count, y);
							printf("  ");
							gotoxy(x + count, y);
						}
					}
				}
				else
				{
					if (count > 0)
					{
						--count;

						gotoxy(x, y);
						printf("%s", buf);
						gotoxy(x+count, y);
					}
				}
				break;

			case ESC:
				if(typ != (NUM | KEYS))
				{
					if (count > 0) {
						memset(buf, ' ', count);
						buf[count] = '\0';
						gotoxy(x, y);
						printf("%s", buf);
						gotoxy(x, y);
						count = 0;
					} else {
						cursor(OFF);
						setecho(OFF);
						*string = '\0';
						return (CLEAR);
					}
				}
				else
				{
					cursor(OFF);
					setecho(OFF);
					return (CLEAR);
				}
				break;
			case ENT_KEY:
				if(typ != (NUM | KEYS))
				{
					if (count >= (smin - 1)) {
						cursor(OFF);
						setecho(OFF);
						strcpy(string, buf);
						return (KEYS);
					}
				}
				else
				{
					cursor(OFF);
					setecho(OFF);
					strcpy(string, buf);
					return (KEYS);
				}

				break;
			default:
				ok = 0;
				if (count < kmax) {		/* max -1 */
					if (typ & FLOAT)
						ok = ((isdigit(key)) || (key == '.'));
					if ((!ok) && (typ & NUM))
						ok = isdigit(key);
					if ((!ok) && (typ & ALPHA))
						ok = ((isalpha(key)) || (key == ' '));
					if (ok) {
						buf[count++] = (char) key;

						if(typ != (NUM | KEYS))
							buf[count] = '\0';

						gotoxy(x, y);
						if (x + count < 14)
							printf("%s", buf);
						else
							printf("%s", buf + (count - 14 + x));

						if(typ == (NUM | KEYS))
						gotoxy(x+count, y);
					}
					break;
				}
				else
				{
					gotoxy(x+kmax, y);
					printf(" ");
					gotoxy(x+kmax, y);
				}
			}
		}
	}
}

// general wait-key function
void wait_key(void)
{
    printf("press any key...");
    while (getchar()==EOF)
	{   if (powereddown())
			backlight(OFF);
		idle();
	}
}

void main_xmenu(void)
{
    static char myheader[]="  MAIN MENU  ";
	static struct xmenus mymenu[] =
	{
		"1 Read tags   ", read_main,
		"2 Write tags  ", write_main,
		"3 Get lockstat", read_lockstatus_main,
		"4 Lock block  ", lock_block_main,
		"5 I-Code      ", icode_xmenu,
		"6 Tag-It      ", tagit_xmenu,
		"7 ISO-15693   ", iso15693_xmenu,
		"8 Test modes  ", test_xmenu,
		"9 Versions    ", show_versions
	};

	int i;
    for (;;)
    {
        i=xmenu(myheader, mymenu, sizeof (mymenu)/sizeof(struct xmenus)-1);
        if (i<0)
            return;
        mymenu[i].itemfunc();
    }
}

void icode_xmenu(void)
{
    static char myheader[]=" I-Code MENU";
	static struct xmenus mymenu[] =
	{
		"1 Quiet mode  ", icode_reset_quiet_mode,
        "2 Family code ", icode_family_appl
	};

	int i;
    for (;;)
    {
        i=xmenu(myheader, mymenu, sizeof (mymenu)/sizeof(struct xmenus)-1);
        if (i<0)
            return;
        mymenu[i].itemfunc();
    }
}

void tagit_xmenu(void)
{
    static char myheader[]=" Tag-it MENU";
	static struct xmenus mymenu[] =
	{		
		"1 Put/Lock    ", tagit_put_lock,
	};

	int i;
    for (;;)
    {
        i=xmenu(myheader, mymenu, sizeof (mymenu)/sizeof(struct xmenus)-1);
        if (i<0)
            return;
        mymenu[i].itemfunc();
    }
}

void iso15693_xmenu(void)
{
    const char myheader[]="  ISO15693  ";
	const struct xmenus mymenu[] =
	{
		"1 AFI (W/L)   ", iso15693_AFI,
		"2 DSFID (W/L) ", iso15693_DSFID,
	};

	int i;
    for (;;)
    {
        i=xmenu(myheader, mymenu, sizeof (mymenu)/sizeof(struct xmenus)-1);
        if (i<0)
            return;
        mymenu[i].itemfunc();
    }
}

void test_xmenu(void)
{
    const char myheader[]="  TEST MENU ";
	const struct xmenus mymenu[] =
	{
		"1 Test mode 1 ", test_mode1,
		"2 Test mode 2 ", test_mode2,
	};

	int i;
    for (;;)
    {
        i=xmenu(myheader, mymenu, sizeof (mymenu)/sizeof(struct xmenus)-1);
        if (i<0)
            return;
        mymenu[i].itemfunc();
    }
}

void read_main(void)
{
	static char tagcode[128];
	int c,x;
	int rfidtype;

	resetkey();
	cursor(NOWRAP);
	setfont(LARGE_FONT,NULL);
	printf("\f");

	gotoxy(0,3);

	printf("Read Tag...");

	for (;;) {
		gotoxy(0,0);						// Wait for trigger key
		for(;;)
		{	c=getchar();
			if (c==TRIGGER_KEY)
				break;
			else if (c==ESC_KEY)
			{	tagpower(OFF);
				return;// 0;
			}
		}

		starttimer(250);

		if (tagpower(ON)==OFF)
            return;// (NOT_OK);

		for(;;)
		{   if (getchar()==ESC_KEY)
			{   tagpower(OFF);
				return;// 0;
			}

            rfidtype = x = AutoTagTypeDetect(tagcode);

			if(rfidtype<0) {
				if (x!=ERROR)
					break;
				if (endtimer())
					break;
				continue;
			}

			x=read_tag(rfidtype, tagcode);

			if (x!=ERROR)
				break;
			if (endtimer())
				break;
		}

		tagpower(OFF);

		if (x==ERROR)
			continue;

		show_taginfo(rfidtype, tagcode, NULL);
	}
}

void write_main(void)
{
	static char tagcode[64];
	int c,x;
	int rfidtype;

	int result;
	int code_id;
	int readonly;

	resetkey();
	cursor(NOWRAP);

	printf("\f");

	setfont(SMALL_FONT,NULL);
	printf("\nPress '.' to set\nthe write data\n  (%s)",write_data);

	setfont(LARGE_FONT,NULL);
	gotoxy(0,3);

	printf("Write Tag...");

	for (;;) {
		gotoxy(0,0);						// Wait for trigger key
		for(;;)
		{	c=getchar();
			if (c==TRIGGER_KEY)
				break;
			else if (c=='.')
			{
				resetkey();

				memset(write_data, '\0', 8);
				
				printf("\fEnter new write\ndata: %s", write_data);
				
				result = get_string(write_data, &code_id, NUM|ALPHA|KEYS, 1, 8, 8, 6, 1);

				resetkey();
				cursor(NOWRAP);

				setfont(SMALL_FONT,NULL);
				printf("\fPress '.' to set\nthe write data\n  (%s)",write_data);

				setfont(LARGE_FONT,NULL);
				gotoxy(0,3);
				printf("Write Tag...");

				continue;
			}
			else if (c==ESC_KEY)
			{	resetkey();
				tagpower(OFF);
				return;// 0;
			}
		}

		starttimer(250);

		if (tagpower(ON)==OFF)
            return;// (NOT_OK);

		readonly = -1;

		for(;;)
		{   if (getchar()==ESC_KEY)
			{
				tagpower(OFF);
				return;// 0;
			}

            rfidtype = x = AutoTagTypeDetect(tagcode);

			if(rfidtype<0) {
				if (x!=ERROR)
					break;
				if (endtimer())
					break;
				continue;
			}

			if(readonly == -1)
			{
				x = write_tag(rfidtype, write_data);
				
				if(x!=ERROR)
					readonly = rfidtype;
			}

			x = ERROR;

			if (readonly>=0)
				x=read_tag(rfidtype, tagcode);

			if (x!=ERROR)
				break;

			if (endtimer())
				break;
		}

		tagpower(OFF);

		if (x==ERROR)
			continue;

		show_taginfo(rfidtype, tagcode, write_data);
	}
}

void read_lockstatus_main(void)
{
	static char tagcode[16];
	int c,x,startpos;
	int rfidtype;
	unsigned char nr_of_blocks;

	resetkey();
	cursor(NOWRAP);

	printf("\f");
	gotoxy(0,3);

	printf("Read Lock status");

	for (;;) {
		gotoxy(0,0);						// Wait for trigger key
		for(;;)
		{	c=getchar();
			if (c==TRIGGER_KEY)
				break;
			else if (c==ESC_KEY)
			{	resetkey();
				tagpower(OFF);
				setfont(LARGE_FONT,NULL);
				return;// 0;
			}
		}

		starttimer(250);

		if (tagpower(ON)==OFF)
            return;// (NOT_OK);

		for(;;)
		{   if (getchar()==ESC_KEY)
			{
				resetkey();
				tagpower(OFF);
				setfont(LARGE_FONT,NULL);
				return;// 0;
			}

			startpos = 0;

            rfidtype = x = AutoTagTypeDetect(tagcode);

			if(rfidtype<0)
			{
				if (x!=ERROR)
					break;
				if (endtimer())
					break;
				continue;
			}

			if (rfidtype==ICODE) //I-code
				nr_of_blocks = 16;
			else if (rfidtype==TAGIT)// TAG-IT
				nr_of_blocks = 8;
			else //iso15693
				nr_of_blocks = tagcode[10]+1;

			x=read_lockstatus(rfidtype,nr_of_blocks);

			if (x!=ERROR)
				break;

			if (endtimer())
				break;
		}

		tagpower(OFF);

		if (x==ERROR)
			continue;

		sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, NULL);
	}
}

void lock_block_main(void)
{
	static char tagcode[16];
	int c,x,startpos;
	int rfidtype;

	resetkey();
	cursor(NOWRAP);

	gotoxy(0,3);
	printf("\fLock blocks\ntest will render\ntag unusable!!!\n\n'9' to continue");
    while ((c=getchar())==EOF);

    if (c!='9')
    {
        printf("\nTest aborted\n");
        wait_key();
        return;
    }

	printf("\fSelect block\nthat must\nbe locked\n");	//select section of 8 blocks to lock
	printf("Press '0'..'7'");

    while ((c=getchar())==EOF);
    
	if (c<'0' || c>'7')
    {
        printf("\nTest aborted\n");
        wait_key();
        return;
    }

	startpos = c - '0';

	printf("\fLock blocks\n");

	for (;;) 
	{
		gotoxy(0,0);						
		for(;;)
		{	c=getchar();
			if (c==TRIGGER_KEY)  // Wait for trigger key
				break;
			else if (c==ESC_KEY)
			{	resetkey();
				tagpower(OFF);
				setfont(LARGE_FONT,NULL);
				return;// 0;
			}
		}

		starttimer(250);

		if (tagpower(ON)==OFF)
            return;// (NOT_OK);

		for(;;)
		{   if (getchar()==ESC_KEY)
			{
				resetkey();
				tagpower(OFF);
				setfont(LARGE_FONT,NULL);
				return;// 0;
			}

            rfidtype = x = AutoTagTypeDetect(tagcode);

			if(rfidtype<0)
			{
				if (x!=ERROR)
					break;
				if (endtimer())
					break;
				continue;
			}

			x=lock_block(rfidtype, startpos);

			if (x!=ERROR)
				break;

			if (endtimer())
				break;
		}

		tagpower(OFF);

		if (x==ERROR)
			continue;

		if(x==OK)
			sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, NULL);
		else if(x==LOCKED)
			sound(TSTANDARD, VHIGH, SLOW, SMEDIUM, SHIGH, SLOW, SMEDIUM, SHIGH, SMEDIUM, SLOW, NULL);
	}
}

int read_tag(int rfidtype, char *bufp)
{
	unsigned int i;
    char dummy;

	int startP;

	startP = 0;

	if(rfidtype==ICODE)
	{
		 //4 version bytes are already placed in bufp by AutoTagTypeDetect()

		if( rfid_ic_getblocks(bufp+4,1,15)<0)
			return (ERROR);
	}
	else if(rfidtype==TAGIT)
	{
		 //8 version bytes are already placed in bufp by AutoTagTypeDetect()
		
		if (rfid_ti_getblocks(bufp+8, 0, 8, 4, &dummy)<0)
			return (ERROR);
	}
	else
	{
		//13 bytes of systeminfo are already placed in bufp by AutoTagTypeDetect()
		
		//read blocks max. of 4 at a time using the read multiple block command
		for (i=startP; i<7; i++) {		//  max = 16x4x4 blocks of data is 256 bytes
		   if (rfid_iso_getblocks_fast(bufp+13+16*i, i*4, 4, 4, &dummy)<0)
				return (ERROR);
		   else
			   ++startP;
		}
	}

	return (OK);
}

int write_tag(int rfidtype, char* data)
{
    static unsigned char buf[4];
    static unsigned char rbuf[4];	//stores icode acces bits
	static char options[5];

	unsigned char error;
	int i,j,pos;
    int err=0;
	int start, nrofblcks;
	int ic_access_bits;

	if(rfidtype==ICODE)
		printf("\fI-Code");
	else if(rfidtype==TAGIT)
		printf("\fTag-it");
	else
		printf("\fISO15693");

	printf(" detected\nStart writing\n");
	
	pos = 0;

	if (rfidtype==ICODE) //i-code
	{
		ic_access_bits = FALSE;
		start = 5;
		nrofblcks = 11;
	}
	else if (rfidtype==TAGIT)
	{
		start = 0;
		nrofblcks = 8;
	}
	else //ISO15693
	{
		start = 0;
		nrofblcks = 28;
	}
	

	i = 0;

	starttimer(250);
	resetkey();

	for(;;)
    {
        if (getchar()==ESC_KEY)
		{
			return ERROR;
		}

		for (j=0; j<4; j++)	//Copy data into write buffer
			buf[j] = data[(pos++)%strlen(data)];

		if (rfidtype==ICODE)
		{
			if(ic_access_bits == FALSE && rfid_ic_getblocks(rbuf,2,1) == 0)
				ic_access_bits = TRUE;

			if(ic_access_bits == TRUE)
			{
				if(((rbuf[(i+start)>>2] >> (((i+start)&3)<<1)) & 3) != 0) // if not locked
				{
					if( (err=rfid_ic_putblock(buf, i+start))!=0 )
						continue;
				}
			}
			else
				continue;
		}
		else if (rfidtype==TAGIT) // TAG-IT
		{
			err=rfid_ti_putblock(buf, i+start, 4, &error);

			if(err!=0 && (err!=-3 || error<0x10 || error>0x12))
				continue;
		}
		else //iso15693
		{
			if( rfidtype == ISO15693_OMRON )	//Set option bit for writing of ISO15693 Philips OMRON tags
			{
				options[0] = 0x01;
				rfidsetoptions(options);
			}
			
			err=rfid_iso_putblock (buf, i+start, 4, &error);

			if( rfidtype == ISO15693_OMRON )	//Clear option bit after writing of ISO15693 Philips OMRON tags
			{
				options[0] = 0x00;
				rfidsetoptions(options);
			}
		
			//if block is locked or not present so skip this block
			if(err!=0 && (err!=-3 || error<0x10 || error>0x12))
				continue;
		}

		gotoxy(0,3);
		++i;
		printf("%02d bytes written", i*4);
		starttimer(100);

		if(i>=nrofblcks)
			break;

		if (endtimer())
		{
			return ERROR;
		}
    }

	starttimer(150);
	return OK;
}

int read_lockstatus(int rfidtype, unsigned char nr_of_blocks)
{
    static unsigned char buf[32];

    unsigned char error;
    int x, i, j, k;

	setfont(TINY_FONT,NULL);

	if(rfidtype==ICODE)
		printf ("\fI-Code lock status:\n(0 = Unlocked)");
	else if(rfidtype==TAGIT)
		printf ("\fTag-it lock status\n(0 = Unlocked)\n");
	else
		printf("\fISO15693 lockstatus:\n");

	starttimer(150);

	j=0;

	for(;;)
	{
		if (getchar()==ESC_KEY)
		{
			resetkey();
			setfont(LARGE_FONT,NULL);
		    return ERROR;
		}

		if(rfidtype==ICODE)
		{
			x=rfid_ic_getblocks(buf,2,1);

			if(x>=0)
			{
				//show all statusses
				for(x=0;x<nr_of_blocks;++x)
				{
					if(x%16==0)
						printf("\n\nBlk %02d - %02d:\n",x,x+15);

					if( ((buf[x>>2] >> ((x&3)<<1)) & 3) == 0) // if locked
						printf("1");
					else
						printf("0"); //not locked
				}
			}
			
			if(x>=0)
				break;
		}
		else if(rfidtype==TAGIT)
		{
			for(i=j;i<nr_of_blocks;++i)
			{
				if((x=rfid_ti_lockstat(buf, i, 4, &error))>=0)
				{
					if(i==0)
						printf("\nBlk %02d - %02d:\n\n",0,nr_of_blocks);

					printf("%c ", '0'+(buf[0]&3));
					++j;
				}
				else
					break;
			}

			if(i==nr_of_blocks)
				break;
		}
		else
		{
			for(i=j;i<((nr_of_blocks-1)/8)+1;++i)
			{
				if((x=rfid_iso_lockstats(buf, i*8, ((((i+1)*8)<=nr_of_blocks)?(8):((nr_of_blocks)-(i*8))), &error))>=0)
				{
					++j;

					if(i%2==0)
						printf("Blk %02d - %02d:\n",i*8,(((i*8)+15 <= nr_of_blocks-1) ? ((i*8)+15) : nr_of_blocks-1));

					for(k=0;k< (((i*8)+7 <= nr_of_blocks-1) ? (8) : (nr_of_blocks - (i*8))) ;++k)
						printf("%c", '0'+(buf[k]&3));

					if(i%2==1)
						printf("\n");
				}
				else
					break;
			}

			if(x>=0)
				break;
		}

		if (endtimer())
		{
			x = -4;
			break;
		}
	}

	if (x<0)
	{
		setfont(LARGE_FONT,NULL);
	    return ERROR;
	}

	setfont(LARGE_FONT,NULL);

    return(OK);
}



// ************************************************************************
// tagit_lock: tests the locking of data blocks.
// Note that this test may render the tag unusable!
// ************************************************************************
int lock_block(int rfidtype, unsigned char block_number)
{
    static char options[5];
	unsigned char error;
    int err;
    unsigned char buf[4];

  	starttimer(150);

	for(;;)
	{
		if (getchar()==ESC_KEY)
		{
			err = -1;
			break;
		}

		if(rfidtype==ICODE)
		{
			if( (err=rfid_ic_getblocks(buf,2,1)) == 0)
			{
				if(((buf[block_number>>2] >> ((block_number&3)<<1)) & 3) != 0) // if not locked
				{
					buf[block_number>>2] &= ~(unsigned char)(3 << ((block_number&3)<<1));

					if( (err=rfid_ic_putblock(buf, 2)) == 0)
					{
						if( (err=rfid_ic_getblocks(buf,2,1)) == 0)
						{
							if(((buf[block_number>>2] >> ((block_number&3)<<1)) & 3) == 0) //check if really locked
							{
								printf("\fI-Code\nBlock #%d locked\nsuccessfully\n",block_number );
									return OK;
							}
						}
					}
					else if(err==-2)
					{
						printf("\fCould not write\naccess bits");
						return LOCKED;
					}
				}
				else
				{
					printf("\fI-Code\nBlock #%d is\nalready locked\n",block_number );
					return LOCKED;
				}	
				
				break;
			}
		}
		else if(rfidtype==TAGIT)
		{
			if((err=rfid_ti_lockstat(buf, block_number, 4, &error))>=0)
			{
				if( buf[0] == 0 )
				{
					if ((err=rfid_ti_lockblock(block_number, &error))>=0)
					{
						if ((err=rfid_ti_lockstat(buf, block_number, 4, &error))>=0)
						{
							if( buf[0] != 0)
							{
								printf("\fTag-it\nBlock #%d locked\nsuccessfully\n",block_number );
								return OK;
							}
						}
					}
				}
				else
				{
					printf("\fTag-it\nBlock #%d is\nalready locked\n",block_number );
					return LOCKED;
				}	
				
				break;
			}
		}
		else
		{
			if((err=rfid_iso_lockstats(buf, block_number, 1, &error))>=0)
			{
				if(buf[0] == 0)
				{
					if( rfidtype == ISO15693_OMRON )	//Set option bit for writing of ISO15693 Philips OMRON tags
					{
						options[0] = 0x01;
						rfidsetoptions(options);
					}

					err=rfid_iso_lockblock(block_number, &error);

					if( rfidtype == ISO15693_OMRON )	//Clear option bit for writing of ISO15693 Philips OMRON tags
					{
						options[0] = 0x00;
						rfidsetoptions(options);
					}
					
					if (err>=0)
					{
						if((err=rfid_iso_lockstats(buf, block_number, 1, &error))>=0)
						{
							if( buf[0] != 0)
							{
								printf("\fISO-15693\nBlock #%d locked\nsuccessfully\n",block_number);
								return OK;
							}
						}
					}
				}
				else
				{
					printf("\fISO-15693\nBlock #%d is\nalready locked\n",block_number);
					return LOCKED;
				}	
				
				break;
			}

		}

		if (endtimer())
		{
			err = -4;
			break;
		}
	}

	return ERROR;
}

int AutoTagTypeDetect(char *bufp) {
	int res;
	if( (res=rfid_ic_getblocks(bufp,0,1))>=0)
		return ICODE;
	if( (res=rfid_ti_version(bufp))>=0)
		return TAGIT;
	if( (res=rfid_iso_systeminfo(bufp))>=0)
	{
		if(bufp[6] != 0x04) //check manufacturer code of ISO-15693 to determine of the write option bit must be set
			return ISO15693; // Default
		else
			return ISO15693_OMRON; // ISO-15693 PHILIPS OMRON TAGS
	}

	return ERROR;
}

int tagpower(int onoff)
{
    if (onoff==ON)
    {
        if (rfidopen()<0)
            return (OFF);
        goodreadled(RED,-1);		// Red LED on
        return (ON);
    }
    rfidclose();

    goodreadled(RED,0);		    // Red LED off
}

int show_taginfo(int rfidtype, char *tagcode, unsigned char *write_data)
{
	int c,x,y,startpos, size_of_data;

	printf("\f");
	setfont(TINY_FONT,NULL);

	if (rfidtype==ICODE)
    {
		printf("TYPE: I-Code        ");
		printf("\nUID: ");
		for (x=7; x>=0; --x)
			printf("%02X", tagcode[x]);
		printf("\nFamily code: %02X     " , tagcode[16]);
		printf("\nApplication ID: %02X  \n" , tagcode[17]);
		startpos = 20;
		size_of_data = 44;
    }
    else if(rfidtype==TAGIT)
    {
		printf("TYPE: Tag-it         ");
		printf("\nUID: %02X%02X%02X%02X        ",tagcode[0],tagcode[1],tagcode[2],tagcode[3]);
		printf("\nChip type: %02d.%02d",(tagcode[4]>>1),tagcode[5]);
		printf("\nMemory format:%2d*%-02.02d\n",(tagcode[6]+1)&0x1F,tagcode[7]+1);
		startpos = 8;
		size_of_data = 32;
    }
	else //iso15693
	{
	    printf("TYPE: ISO15693-Tag   ");
		printf("\nUID: ");
		for (x=7; x>=0; --x)
			printf("%02X", tagcode[x]);

		printf("\nDSFID: %02X   AFI: %02X  ", tagcode[8], tagcode[9]);
		printf("\nMemory format:%02d * %-02.02d \n",tagcode[10]+1,tagcode[11]+1);
		startpos = 13;
		size_of_data = 112;
    }

	printf("Data: ");

	c=OK;

	for(x=y=6;x<6+size_of_data;x++)
	{
		if (y==20)
		{
			putchar('\n');
			y=0;
		}

		if(write_data==NULL)
			if( tagcode[(x-6)+startpos] >= 0x20 && tagcode[(x-6)+startpos]<=0x7E)
				printf("%c",tagcode[(x-6)+startpos]);
			else
				printf(".");
		else
		{
			if(tagcode[(x-6)+startpos] == write_data[(x-6)%strlen(write_data)])
				printf("%c",tagcode[(x-6)+startpos]);
			else
			{
				if( tagcode[(x-6)+startpos] >= 0x20 && tagcode[(x-6)+startpos]<=0x7E)
					printf("\tr%c\tr",tagcode[(x-6)+startpos]);
				else
					printf("\tr.\tr");
				
				c=ERROR;
			}
		}

		y++;
	}

	if(c==ERROR)
		sound(TSTANDARD, VHIGH, SLOW, SMEDIUM, SHIGH, SLOW, SMEDIUM, SHIGH, SMEDIUM, SLOW, NULL);
	else
		sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, NULL);

	setfont(LARGE_FONT,NULL);
}


// ************************************************************************
// icode_reset_quiet_mode: sends the reset_quiet command (in case the tag was already in
// quiet mode), then continues with the following test:
// sets the tag to quiet mode, checks this by trying to read it (which should fail),
// then sends the reset_quiet command and checks success by again reading it (this time
// it should read).
// ************************************************************************
void icode_reset_quiet_mode(void)
{
    int err;
    int error=0;
    unsigned char buf[5];

    printf("\fI-Code Quiet\n");
    if (tagpower(ON)==OFF)
    {
        printf("ERROR: poweron\n failed\n");
        wait_key();
        return;
    }
    if (rfid_ic_resetquietbit()!=0)        // reset quiet bit, just in case
    {
        printf("general ERROR 1\n");
        tagpower(OFF);
        wait_key();
        return;
    }

    err=rfid_ic_getblocks(buf, 3, 1);   // get block 3
    if (err==-1)
    {
        printf("general ERROR 2\n");
        tagpower(OFF);
        wait_key();
        return;
    }
    if (err==-2)
    {
        printf("no tag? ERROR 2\n");
        tagpower(OFF);
        wait_key();
        return;
    }
    buf[0]|=0x0C;                       // set QUIET mode bits
    switch (rfid_ic_putblock(buf, 3))       // write block 3 back
    {
        case -1:
            printf("general ERROR 1\n");    // no data or unexpected data received from module
            error++;
            break;
        case -2:
            printf("no tag? ERROR 1\n");    // module reported "no tag in range"
            error++;
            break;
        case -3:
            printf("verify ERROR 1\n");     // no verify
            error++;
            break;
        case 0:
            printf("verify OK????\n");      // that WOULD be very strange!
            error++;
        case -4:
                                            // module should NOT be able to verify since the
        default:                            // QUIET bits are now set!
            break;
    }
    if (error)
    {
        tagpower(OFF);
        wait_key();
        return;
    }

    err=rfid_ic_getblocks(buf, 3, 1);   // get block 3
    if (err==-1)
    {
        printf("general ERROR 4\n");
        tagpower(OFF);
        wait_key();
        return;
    }
    if (err==0)
    {
        printf("Not QUIET ERROR\n");    // tag should NOT respond now!
        tagpower(OFF);
        wait_key();
        return;
    }
    if (rfid_ic_resetquietbit()!=0)        // reset quiet bit
    {
        printf("general ERROR 5\n");
        tagpower(OFF);
        wait_key();
        return;
    }
    err=rfid_ic_getblocks(buf, 3, 1);   // get block 3 (tag should respond again)
    if (err==-1)
    {
        printf("general ERROR 6\n");
        tagpower(OFF);
        wait_key();
        return;
    }
    if (err==-2)
    {
        printf("no tag? ERROR 6\n");
        tagpower(OFF);
        wait_key();
        return;
    }
    printf("Test passed\n");
    tagpower(OFF);
    wait_key();
}

// ************************************************************************
// icode_family_appl: sets the tag to nonzero family code and application ID, and checks
// if the tag will read only when the correct family code/applID is supplied
// (test of set_options function and the correct use of the options)
// ************************************************************************
void icode_family_appl(void)
{
    int i,err;
    int error=0;
    unsigned char buf[5];

    printf("\f\"family/appl\"\n");

    if (tagpower(ON)==OFF)
    {
        printf("ERROR: poweron\n failed\n");
        wait_key();
        return;
    }

    for (i=0; i<4; i++)
        buf[i]=0;
    buf[0]=1;  // family code
    buf[1]=2;  // application code
    switch (rfid_ic_putblock(buf, 4))  // write block 4
    {
        case -1:
            printf("general ERROR 1\n");    // no data or unexpected data received from module
            error++;
            break;
        case -2:
            printf("no tag or locked\n");    // module reported "no tag in range"
            error++;
            break;
        case -3:
            printf("no verify ERROR 1\n");     // no verify
            error++;
            break;
        case -4:
            printf("unable verify 1\n");     // no verify
            error++;
        default:
            break;
    }
    if (error)
    {
        tagpower(OFF);
        wait_key();
        return;
    }
    if (rfidsetoptions(buf)!=0)
    {
        printf("setoptions ERROR\n");
        tagpower(OFF);
        wait_key();
        return;
    }
    err=rfid_ic_getblocks(buf, 4, 1);             // read with familycode and applid as set
    if (err==-1)
    {
        printf("general ERROR 2\n");
        tagpower(OFF);
        wait_key();
        return;
    }
    if (err==-2)
    {
        printf("no tag? ERROR 2\n");
        tagpower(OFF);
        wait_key();
        return;
    }

    buf[0]=2;
    buf[1]=1;                                   // exchange the options

    if (rfidsetoptions(buf)!=0)
    {
        printf("setoptions ERROR\n");
        tagpower(OFF);
        wait_key();
        return;
    }
    err=rfid_ic_getblocks(buf, 4, 1);             // read with familycode and applid
    if (err==-1)
    {
        printf("general ERROR 3\n");
        tagpower(OFF);
        wait_key();
        return;
    }
    if (err==0)
    {
        printf("tag read?ERROR\n");                // should not happen!
        tagpower(OFF);
        wait_key();
        return;
    }

    buf[0]=0;
    buf[1]=0;                                       // set back to 0

    if (rfidsetoptions(buf)!=0)
    {
        printf("setoptions ERROR\n");
        tagpower(OFF);
        wait_key();
        return;
    }

    switch (rfid_ic_putblock(buf, 4))  // write block 4
    {
        case -1:
            printf("general ERROR 4\n");    // no data or unexpected data received from module
            error++;
            break;
        case -2:
            printf("no tag? ERROR 4\n");    // module reported "no tag in range"
            error++;
            break;
        case -3:
            printf("verify ERROR 4\n");     // no verify
            error++;
            break;
        case -4:
            printf("unable verify 4\n");     // no verify
            error++;
        default:
            break;
    }
    if (error)
    {
        tagpower(OFF);
        wait_key();
        return;
    }

    tagpower(OFF);
    printf("Test passed\n");
    wait_key();
}

// ************************************************************************
// tagit_put_lock: tests the writing and locking of data blocks.
// Also tests the lock_status function.
// Note that this test may render the tag unusable!
// ************************************************************************
void tagit_put_lock(void)
{
    unsigned char error;
    int i, j, c, err;
    unsigned char buf[32];

    printf("\ftest will render\ntag unusable!!!\nCLR to abort,\n9 to continue");

    while ((c=getchar())==EOF);
    if (c!='9')
    {
        printf("\nTest aborted\n");
        wait_key();
        return;
    }

    printf("\n");

    if (tagpower(ON)==OFF)
    {
        printf("ERROR: poweron\n failed\n");
        wait_key();
        return;
    }

    for (i=0; i<8; i++)
    {
        for (j=0; j<4; j++)
            buf[j]=j+4*i+' ';
        if ((err=rfid_ti_putandlockblock (buf, i, 4, &error))<0)
        {
            tagpower(OFF);
            if (err==-1)
                printf ("general ERROR\n");
            if (err==-2)
                printf ("tag not in range\n");
            if (err==-3)
                printf ("error code %X\n", error);
            printf ("(block #%i)\n", i);
            wait_key();
            return;
        }
    }
    if ((err=rfid_ti_getblocks (buf, 0, 8, 4, &error))<0)
    {
        tagpower(OFF);
        if (err==-1)
            printf ("general ERROR\n");
        if (err==-2)
            printf ("tag out of range\n");
        if (err==-3)
            printf ("error code %X\n", error);
        wait_key();
        return;
    }
    for (i=0; i<32; i++)
        if (buf[i]!=i+' ')
        {
            tagpower (OFF);
            printf ("not verified\n(block #%i)", i/4);
            wait_key();
            return;
        }
    for (i=0; i<8; i++)
    {
        if ((err=rfid_ti_lockstat(buf, i, 4, &error))<0)
        {
            tagpower(OFF);
            if (err==-1)
                printf ("general ERROR\n");
            if (err==-2)
                printf ("tag out of range\n");
            if (err==-3)
                printf ("gls error #%X\n", error);
            wait_key();
            return;
        }
        if ((buf[0]&3)!=1)
        {
            tagpower(OFF);
            printf("lockstat=%i\n(block #%i)\n", buf[0]&3, i);
            wait_key();
            return;
        }
    }
    printf ("Test Passed!\n");
    tagpower(OFF);
    wait_key();
}

// ************************************************************************
// iso15693_AFI: demonstrates the writing and locking of AFI
// ************************************************************************
void iso15693_AFI(void)
{
	iso15693_AFI_DSFID(1);
}

// ************************************************************************
// iso15693_DSFID: demonstrates the writing and locking of DSFID
// ************************************************************************
void iso15693_DSFID(void)
{
	iso15693_AFI_DSFID(0);
}

// ************************************************************************
// iso15693_AFI_DSFID: demonstrates the writing and locking of AFI and DSFID
// testAFI=1 -> AFI,    testAFI = 0 -> DSFID
// No the library doens't contain a funtion the check the lockstatus of the
// DSFID and the AFI. They can be checked by using the putAFI and the putDSFID
// command and then checking the error response code
// ************************************************************************
void iso15693_AFI_DSFID(int testAFI)
{
    unsigned char error;
    int mode, c, err;
    unsigned char buf[32];

	printf("\fChoose option:\n");	//choose between, writing, locking and reading the status
	printf("-Write,  press 0\n");	//of the AFI or DSFID
	printf("-Lock,   press 1\n");
	printf("-Status, press 2");

    while ((c=getchar())==EOF);
    if (c<'0' || c>'2')
    {
        printf("\nTest aborted\n");
        wait_key();
        return;
    }
	printf("\n");

	mode = (int)c - '0';		//set mode (Write/lock/status)

	if(mode==1)
	{
		printf("\ftest will render\ntag unusable!!!\nCLR to abort,\n9 to continue");

		while ((c=getchar())==EOF);
		if (c!='9')
		{
			printf("\nTest aborted\n");
			wait_key();
			return;
		}
		printf("\n");
	}

	if (tagpower(ON)==OFF)
    {
        printf("ERROR: poweron\n failed\n");
        wait_key();
        return;
    }

	if(mode==0 || mode==2) {	//writing and lock status
		buf[0]=2;

		starttimer(100);

		for(;;)
		{
			if (getchar()==ESC_KEY)
			{   setfont(LARGE_FONT,NULL);
			    tagpower(OFF);
				wait_key();
				return;
			}

			if(testAFI)
				err=rfid_iso_putAFI(buf, &error);		//write AFI
			else 
				err=rfid_iso_putDSFID(buf, &error);		//write DSFID

			if (endtimer()) {
				if(err>=0) err = -1;
				break;
			}

			if(err==-1 || err==-2 || (mode==2 && err==-3 && error!=0x12))
				continue;

			if(mode==0)
			{
				if(err==0)
					printf ("Write..OK\n");
			}
			else if(mode==2)
			{
				if(err==0)
					printf ("\fStatus: Unlocked\n");
				if(err==-3)
					printf ("\fStatus: Locked\n");
			}
			break;
		}

		if(err==-1 || err==-2 || (mode==0 && err==-3) || (mode==2 && err==-3 && error!=0x12))
		{
			tagpower(OFF);

			if (err==-1)
				printf ("general ERROR\n");
			if (err==-2)
				printf ("tag not in range\n");
			if (err==-3)
				printf ("error code %X\n", error);

			wait_key();
			return;
		}
	}
	else {	//mode==1 -> lock AFI or DSFID

		starttimer(100);

		for(;;)
		{
			if (getchar()==ESC_KEY)
			{   setfont(LARGE_FONT,NULL);
			    tagpower(OFF);
				wait_key();
				return;
			}

			if(testAFI && (err=rfid_iso_lockAFI(&error))==0)
				break;
			else if((err=rfid_iso_lockDSFID(&error))==0)
				break;

			if(err==-3 && error==0x11) //already locked or previous response has been missed
				break;

			if (endtimer()) {
				if(err>=0) err = -1;
				break;
			}
		}

		if(err<0)
		{
			tagpower(OFF);
			if (err==-1)
				printf ("general ERROR\n");
			if (err==-2)
				printf ("tag not in range\n");
			if (err==-3)
				printf ("error code %X\n", error);
			wait_key();
			return;
		}

		printf ("Locking..OK\n");
	}
	sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, NULL);

	printf ("Test Passed!\n");
    tagpower(OFF);
    wait_key();
}

void show_versions(void)
{
	static char os[8+1];
	static char firmware[8+1];
	char *lib;
	
	printf("\fSoftware Version");
	version_os(os);
	os[3] = '\0';
	os[1] = 'F';
	printf("\n   %s%s\n press any key", os,VERSIONSUFFIX);
	while (getchar()==EOF)
	{	if (powereddown())
			backlight(OFF);
		idle();
	}
	resetkey();

	printf("\fLibrary Version ");

	lib = rfid_lib_version();

	printf("\n   %s\n press any key",lib);
	while (getchar()==EOF)
	{	if (powereddown())
			backlight(OFF);
		idle();
	}
	resetkey();

	printf("\fFirmware Version");

	if (tagpower(ON)==OFF)
		printf("\nFirmware ERROR\n");
	else
		if (rfidversion (firmware)<0)
			printf("\nFirmware ERROR\n");
	else
		printf("\n   %s\n", firmware);
	tagpower(OFF);

	printf(" press any key");
	while (getchar()==EOF)
	{	if (powereddown())
			backlight(OFF);
		idle();
	}
	resetkey();
}


// ************************************************************************
// test mode 1: switch on the antenna power until a key is pressed
// ************************************************************************
void test_mode1(void)
{
    printf("\ftest mode 1\n");
    
	if (tagpower(ON)==OFF)
        printf("ERROR: poweron\n failed\n");
    else
    {    
		if (rfid_testmode(1)!=0)
            printf("ERROR: general\n");

		wait_key();
    }

	if (rfid_testmode(0)!=0)
    {
		printf("\nError: general\n");
        wait_key();
    }
    tagpower(OFF);
}

// ************************************************************************
// test mode 2: switch on the antenna power (modulation on) until a key is pressed
// ************************************************************************
void test_mode2(void)
{
    printf("\ftest mode 2\n");
    
	if (tagpower(ON)==OFF)
        printf("ERROR: poweron\n failed\n");
    else
    {
		if (rfid_testmode(2)!=0)
            printf("ERROR: general\nerror\n");
	}

    wait_key();
    tagpower(OFF);
}


void main(void)
{
	strcpy(write_data, "00000000");
	
	resetkey();
	
	for (;;)
	{
        main_xmenu();
	}
}
