/**************************************************************************************/
/*		Demo application: Software upgrade and transmit barcode data via FTP          */
/*      for the PHL2700, PHL1300, OPL972x and the OPH100x                             */
/**************************************************************************************/
/* File Names :                                                                       */
/*   main.c		: Main application file (Barcode reading, configuration)              */
/*   upgrade.c  : Parses a stored FTP directory listing for software updates          */
/*   comm.c     : FTP communication, transmit data & searches for a software update   */
/*   internet.h : Include file for the TCP/IP library                                 */
/*   tp_info.c  : (Used only by OPH to set internal application version)              */
/* 										                                              */
/**************************************************************************************/
/* Description                                                                        */
/*   - Reads barcode and stores them with time & data in fixed length database file   */
/*   - Allows you to configure all ISP/FTP communication settings using special       */
/*     configuration barcodes:													      */
/*			+ISPNR<ISP phone nr>													  */
/*			+ISPUS<ISP Username>													  */
/*			+ISPPA<ISP Password>													  */
/*			+COMPR<COM-Port> ('0'=cable, '2'=cradle)	                              */
/*			+FTPDN<FTP address>													      */
/*			+FTPIP<FTP IP-addr													      */
/*			+FTPUS<FTP Username>													  */
/*			+FTPPA<FTP Password>													  */
/*     (Note: All settings are stored in flash, so only have to be configured once)   */
/*   - Scan the +-CONN-+ label (as found in the Universal menu-book to start dialing  */
/*     the ISP, connect to FTP, send the scanned barcode data, check for software     */
/*     upates and, if a software update was found, download & install this software   */
/* 										                                              */
/**************************************************************************************/
/* 				Modification Information                                              */
/**************************************************************************************/
/*	  Date	   | Version  |					Contents								  */
/**************************************************************************************/
/*	2006/01/17 | xxx30570 | First release by Ronny de Winter (OSE)                    */
/*	           |          |                                                           */
/**************************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "lib.h"
#include "comm.h"

#define DATA_BASE			"DATA.TXT"
#define VERSION				"30571"

#define CHECK_STRING         "DEMO"

// Structure for special function barcodes
typedef struct
{
	char code[6+1];					// 6-char indentifier of barcode
	char description[13+1];			// Description of variable
	char *data;						// Storage of configuration string
} s_special_bcr;


// Table for special function barcodes
const s_special_bcr special[] = 
{
	{ "+ISPNR", "ISP phone nr", ftp_setup.isp_nr  	 },
	{ "+ISPUS", "ISP Username", ftp_setup.isp_user	 },
	{ "+ISPPA", "ISP Password", ftp_setup.isp_pass	 },
	{ "+COMPR", "COM-Port",	    ftp_setup.isp_com	 },
	{ "+FTPDN", "FTP address",  ftp_setup.ftp_domain },
	{ "+FTPIP", "FTP IP-addr",  ftp_setup.ftp_ip  	 },
	{ "+FTPUS", "FTP Username", ftp_setup.ftp_user	 },
	{ "+FTPPA", "FTP Password", ftp_setup.ftp_pass	 },
};

// Barcode record defines
#define BCR_MAX		(60-1)

#define SZ_BARCODE	BCR_MAX
#define SZ_DATE		10
#define SZ_TIME		8
#define SZ_RECORD	( SZ_BARCODE +1+SZ_DATE+1+SZ_TIME+2 )

// Barcode struct
typedef struct
{
	char barcode[ SZ_BARCODE +1 ];
	char time[ SZ_TIME+1 ];
	char date[ SZ_DATE+1 ];
}db_record;

/****************************************************************************/
#if PHL|PHL1000|PHL2700
/****************************************************************************/
#define FLASH_BLOCK_NUMBER      1               // Do NOT change this value!!!
#define FLASH_ADDRESS           0x204001UL;     // Do NOT change this value!!!
extern void erase_sector(int no);
#define eraseblock(no) erase_sector((unsigned int)no)
extern void write_flash(unsigned char *address, unsigned char data);
void programbuffer(unsigned char *b, unsigned char *a, int len) { int i; for(i=0;i<len;++i) write_flash(a+i, *(b+i)); }
/****************************************************************************/
#elif OPH
/****************************************************************************/
#define FLASH_BLOCK_NUMBER      30              // Do NOT change this value!!!
#define FLASH_ADDRESS           0x1C1E0000UL;   // Do NOT change this value!!!
/****************************************************************************/
#else    // OPL972x
/****************************************************************************/
#define FLASH_BLOCK_NUMBER      7               // Do NOT change this value!!!
#define FLASH_ADDRESS           0xC70000UL;     // Do NOT change this value!!!
#endif
/****************************************************************************/

// Flash functions
void reset_settings_in_flash(void);
void store_settings_in_flash(void);
int read_settings_from_flash(void);

// Barcode handling functions 
int handle_scanned_barcode(char *buffer);
void scan_and_communicate(void);
void display_scan_barcode(db_record *db_rec);
void create_db_rec_struct(char* barcode, db_record *db_rec);
void save_data(db_record *db_rec);

// Buzzer functions
void info_sound(void)	{	sound( TSHORT, VMEDIUM, SHIGH, SLOW, SHIGH, SHIGH, SLOW, 0);	}
void okbeep(void)		{	sound(TSTANDARD,VMEDIUM,SHIGH,SMEDIUM,SHIGH,0);					}
void errbeep(void)		{	sound(TLONG,VMEDIUM,SLOW,SMEDIUM,SHIGH,0);						}
//

/**************************************************************************************/
/* reset_settings_in_flash                                                            */
/**************************************************************************************/
/* Description                                                                        */
/*   Clears all communication settings in flash (and in RAM)                          */
/**************************************************************************************/
void reset_settings_in_flash(void)
{
	int max;
	int count;

	count = 0;
	max = sizeof( special ) / sizeof( s_special_bcr );

	for(count = 0; count < max; ++count)
		special[count].data[0] = '\0';

	store_settings_in_flash();
}

/**************************************************************************************/
/* store_settings_in_flash                                                            */
/**************************************************************************************/
/* Description                                                                        */
/*   Store all settings that are configured in the special function table in Flash    */
/**************************************************************************************/
void store_settings_in_flash(void)
{
	unsigned char *address;
	int max;
	int count;

	eraseblock(FLASH_BLOCK_NUMBER);

	address = (unsigned char *)FLASH_ADDRESS;
	
    // Store check string in front of the settings
    programbuffer((unsigned char *)CHECK_STRING, address, strlen(CHECK_STRING));
    address += strlen(CHECK_STRING);

	
	count = 0;
	max = sizeof( special ) / sizeof( s_special_bcr );

	while( count < max )
	{
		programbuffer((unsigned char *)special[count].code, address, strlen(special[count].code)+1);
		address += strlen(special[count].code) + 1;
		programbuffer((unsigned char *)special[count].data, address, strlen(special[count].data)+1);
		address += strlen(special[count].data) + 1;
		count++;
	}
}

/**************************************************************************************/
/* read_settings_from_flash                                                           */
/**************************************************************************************/
/* Description                                                                        */
/*   Restores all settings of the special function table from Flash                   */
/*                                                                                    */
/* Returns                                                                            */
/*     -1           :   ERROR, Unexpected data found, settings not restores           */
/*      0           :   Settings succesfully loaded                                   */
/**************************************************************************************/
int read_settings_from_flash(void)
{
	unsigned char *address;
	char *ptr;
	int max;
	int i, count;
	
	address = (unsigned char *)FLASH_ADDRESS;
	
    // Verify that check string is found is flash
    for(i=0; i < strlen(CHECK_STRING); ++i)
    {
        if (*address++ != CHECK_STRING[i] )
            return (-1);           // Check failed, return ERROR
    }

	count =0;
	max = sizeof( special ) / sizeof( s_special_bcr );

	while( count < max )
	{
		ptr = (char *)special[count].code;
		
		while (*ptr == *address)
		{
			if(*ptr == '\0')
				break;

			++ptr;
			++address;
		}

		if(*ptr != '\0')
			return -2;

		ptr = (char *)special[count].data;

		++address;

		while ((*ptr = *address) != 0)
	    {
		    ++ptr;
			++address;
		}

		++address;

		*ptr = '\0';

		count++;
	}

	return 0;
}

/**************************************************************************************/
/* display_scan_barcode                                                               */
/**************************************************************************************/
/* Description                                                                        */
/*   Prints the scanned barcode and time on the display                               */
/*                                                                                    */
/* Input                                                                              */
/*   db_record *db_rec    :  Barcode structure                                        */
/**************************************************************************************/
void display_scan_barcode(db_record *db_rec)
{
	gotoxy(0,0);
	printf("\tr SCAN BARCODE   \tr");
	gotoxy(0,1);
	
	if(db_rec->barcode[0] != '\0')
	{
		printf("%-18.18s", db_rec->barcode);
		gotoxy(0,2);
		printf("%-8.8s", db_rec->time );
		gotoxy(0,3);
		printf("%-10.10s", db_rec->date); /* clear line */
	}

	gotoxy(0,1);
}

/**************************************************************************************/
/* create_db_rec_struct                                                               */
/**************************************************************************************/
/* Description                                                                        */
/*   Created a fixed length barcode record structure                                  */
/*                                                                                    */
/* Input                                                                              */
/*    char* barcode       :  Barcode string                                           */
/*                                                                                    */
/* Output                                                                             */
/*   db_record *db_rec    :  Barcode record structure                                 */
/**************************************************************************************/
void create_db_rec_struct(char* barcode, db_record *db_rec)
{
	static struct date d;
	static struct time t;

	getdate( &d );
	gettime( &t );

	sprintf( db_rec->barcode, "%-*.*s",	SZ_BARCODE, SZ_BARCODE, barcode );
	sprintf( db_rec->time, "%02d:%02d:%02d", t.ti_hour, t.ti_min, t.ti_sec );
	sprintf( db_rec->date, "%02d/%02d/%04d", d.da_day, d.da_mon, d.da_year );
}

/**************************************************************************************/
/* create_db_rec_struct                                                               */
/**************************************************************************************/
/* Description                                                                        */
/*   Appends the barcode record to the datbase file                                   */
/*                                                                                    */
/* Input                                                                              */
/*   db_record *db_rec    :  Barcode record structure that is to be stored            */
/*                                                                                    */
/* Input (Global)                                                                     */
/*   filename (DATA_BASE) :  File in which the record is stored                       */
/**************************************************************************************/
void save_data(db_record *db_rec)
{
	int fd;
	static char record[ SZ_RECORD + 1 ];

	if( coreleft() < 7000L )
	{
		errbeep();  

		if( coreleft() < 5000L )
			return;		
	}

	sprintf(record, "%-*.*s,%-*.*s,%-*.*s\r\n", SZ_BARCODE, SZ_BARCODE, db_rec->barcode,
				SZ_TIME, SZ_TIME, db_rec->time,
				SZ_DATE, SZ_DATE, db_rec->date );

	if ((fd = open( (char*) DATA_BASE, O_RDWR |O_CREAT |O_APPEND |O_BINARY,0x0)) == -1)
	{
		errbeep();
		return;
	}

	if( write( fd, record, SZ_RECORD ) != SZ_RECORD )
	{
		errbeep();
	}

	close( fd );
}

/**************************************************************************************/
/* handle_scanned_barcode                                                             */
/**************************************************************************************/
/* Description                                                                        */
/*   Handles the scanned barcode and checks if the label is configuration label       */
/*                                                                                    */
/* Input                                                                              */
/*   char *barcode    :  Contains the scanned barcode                                 */
/*                                                                                    */
/* Returns                                                                            */
/*     -1           :   Not a valid configuration label, so handled as data           */
/*      0           :   Configuration label, setting is stored in settings table      */
/**************************************************************************************/
int handle_scanned_barcode(char *barcode)
{
	int max;
	int count;
	
	count = 0;
	max = sizeof( special ) / sizeof( s_special_bcr );
	while( count < max )
	{
		// Check if the label matches a valid configuration label
		if( strncmp( special[count].code, barcode, strlen( special[count].code )) == 0 )
		{
			strcpy(special[count].data, &barcode[6]);
			return count;
		}
		count++;
	}

	return -1;
}

/**************************************************************************************/
/* ScanBarcode                                                                        */
/**************************************************************************************/
/* Description                                                                        */
/*   Barcode reading routine (single read, 4 seconds)                                 */
/*                                                                                    */
/* Input                                                                              */
/*   int min_length :   Minimum allowed barcode length                                */
/*   int max_length :   Maximum allowed barcode length                                */
/*                                                                                    */
/* Output                                                                             */
/*   char *barcode  :   Storage location for the scanned barcode                      */
/*                                                                                    */
/* Returns                                                                            */
/*     -1           :   Not a valid configuration label, so handled as data           */
/*      0           :   Configuration label, setting is stored in settings table      */
/**************************************************************************************/
int ScanBarcode( char* barcode, int min_length, int max_length)
{
	int key;
	struct barcode code = {0};

	resetkey();
	code.min = min_length;
	code.max = max_length;
	code.text = barcode;
	scannerpower( SINGLE, 200 );

	while( readbarcode( &code ) != OK )
	{
		if( kbhit() )
		{
			if(( key = getchar() ) != TRIGGER_KEY )
			{
				// put the character back in the input buffer
				// so it can be handled by another function
				ungetc( key, stdin);
				scannerpower( OFF, 0);
				return( ERROR );
			}
		}
		idle();
	}

	goodreadled( GREEN, 10 );
	//scannerpower( OFF, 0);

	return( OK );
}



/**************************************************************************************/
/* scan_and_communicate                                                               */
/**************************************************************************************/
/* Description                                                                        */
/*   Reads, executes, stores and displays normal & configuration barcodes             */
/*                                                                                    */
/*   - Scan configuration barcodes to configure the FTP communication parameter       */
/*   - Scan +-CONN-+ to start dialing ISP, connect to FTP, send data, check for       */
/*     software upates and download & install if a software update was found          */
/**************************************************************************************/
void scan_and_communicate(void)
{
	static char barc[ BCR_MAX + 1 ];
	static db_record db_rec;

	int ch, res;

	memset( barc, '\0', sizeof( barc ));
	memset( &db_rec, '\0', sizeof( db_record ));

	putchar('\f');
	for(;;)
	{
		display_scan_barcode( &db_rec );
		
		if( ScanBarcode( barc, 1, BCR_MAX) == OK )
		{
			if((res=handle_scanned_barcode(barc)) >= 0)
			{
				store_settings_in_flash();

				info_sound();
				goodreadled( GREEN, 25 );

				printf("%-12.12s: \n%-16.16s\n                ", special[res].description, special[res].data);

				memset( &db_rec, '\0', sizeof( db_record ));
			}
			else if( strcmp(barc, "+-CONN-+") == 0)
			{
				info_sound();
				goodreadled( GREEN, 25 );

				if(Transmit_and_Update(DATA_BASE, VERSION) != -1)
				{
					while(!kbhit())
						idle();
				}

				putchar('\f');
			}
			else
			{
				okbeep();
				goodreadled( GREEN, 25 );
				create_db_rec_struct( barc, &db_rec );
				save_data( &db_rec );
			}
			continue;
		}

		if( (ch=getchar()) == Q1_KEY || ch == CLR_KEY)
			return;

		if(ch == '.')	// Shortkey for "+-CONN-+"
		{
			goodreadled( GREEN, 25 );

			if(Transmit_and_Update(DATA_BASE, VERSION) != -1)
			{
				while(!kbhit())
					idle();
			}

			putchar('\f');
		}
	}
}

/**************************************************************************************/
/* main                                                                               */
/**************************************************************************************/
/* Description                                                                        */
/*   Application starts here                                                          */
/**************************************************************************************/
void main(void)
{	
	static char current_version[8+1];
	
	systemsetting((const unsigned char *)"YCSZA0");	// Enable menu-labels, 115200bps

	cursor(NOWRAP);

	memset( &ftp_setup, '\0', sizeof( struct ftp_settings ));

	if(read_settings_from_flash() < 0)
	{
		reset_settings_in_flash();
	}

	version_os(current_version);
	current_version[1] = 'F';
	current_version[3] = '\0';
	strcat(current_version, VERSION);

	remove(current_version); // Remove application file, if present

	for(;;)
	{
		printf("\fFTP demo\napplication\nVersion:\n    %s", current_version);
		delay(100);
		scan_and_communicate();
	}
}
