	/* kermit.c - Kermit protocol for the PHL ported by Oscar Jacobse */
/* static char *sccsid = "@(#)Kermit version 3.1(0) 8/24/84";     */
/*
 *  K e r m i t  File Transfer Utility
 *
 *  UNIX Kermit, Columbia University, 1981, 1982, 1983, 1984
 *      Bill Catchings, Bob rCattani, Chris Maio, Frank da Cruz,
 *      Alan Crosswell, Jeff Damens
 *
 *  Also:   Jim Guyton, Rand Corporation
 *          Walter Underwood, Ford Aerospace
 *
 *  usage:  kermit c [lbphe line baud par escape-char]  to connect
 *          kermit s [ddiflbpt line baud par] file ...  to send files
 *          kermit r [ddiflbpt line baud par]           to receive files
 *
 *  where   c=connect, s=send, r=receive,
 *          d=debug, i=image mode, f=no filename conversion, l=tty line,
 *          b=baud rate, e=escape char, h=half duplex, t=turnaround char,
 *          p=parity, z=log packet transactions.
 *
 *  For remote Kermit, format is either:
 *          kermit r                                    to receive files
 *  or      kermit s file ...                           to send files
 *
 */

/*
 *  Modification History:
 *
 *  Aug 84 David Mutterer - Add definition for UNIX System 5 Version 2.
 *
 *  July 84 Bill Catchings and Jeff Damens - Add necessary commands for
 *      communicating with IBM mainframes (t,p,h).  Also started to
 *      make it more modular.  Eventually all conditionals should be
 *      removed.  After that we will go to a LEX version which will
 *      implement all the states needed for a full server version.  A
 *      command parser is also needed.  Limited VAX VMS support was also
 *      added (no connect command).  Link together KERMIT.C, KER%%%.C
 *      (UNX for all present UNIX versions, VMS for VAX VMS) and KERCN%.C
 *      (V for Venix on the Pro and U for UNIX this module not used for
 *      VMS).
 *
 *  May 21 84 - Roy Smith (CUCS), strip parity from checksum in rpack()
 *
 *  Oct. 17 Included fixes from Alan Crosswell (CUCCA) for IBM_UTS:
 *          - Changed MYEOL character from \n to \r.
 *          - Change char to int in bufill so getc would return -1 on
 *            EOF instead of 255 (-1 truncated to 8 bits)
 *          - Added read() in rpack to eat the EOL character
 *          - Added fflush() call in printmsg to force the output
 *          NOTE: The last three changes are not conditionally compiled
 *                since they should work equally well on any system.
 *
 *          Changed Berkeley 4.x conditional compilation flag from
 *              UNIX4X to UCB4X.
 *          Added support for error packets and cleaned up the printing
 *              routines.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "lib.h"
#include "irda_lib.h"

#define MAXPACKSIZ  94			//hack ronny94   /*94 */          /* Maximum packet size */
//#define SOH         1			/* Start of header */
//#define CR          13			/* ASCII Carriage Return */
//#define SP          32			/* ASCII space */
//#define DEL         127			/* Delete (rubout) */
#define XON         17			/* ASCII XON */

#define MAXTRY      3          /* Times to retry a packet */
#define MYQUOTE     '#'			/* Quote character I will use */
#define MYPAD       0			/* Number of padding characters I will need */
#define MYPCHAR     0			/* Padding character I need (NULL) */

#define MYTIME      5           /* Seconds after which I should be timed out */
#define MAXTIM      60			/* Maximum timeout interval */
#define MINTIM      2			/* Minumum timeout interval */
#define KERMIT_TRUE   0           /* Boolean constants */
#define KERMIT_FALSE  -1
#define MYEOL         '\r'


#define tochar(ch)  ((ch) + ' ')
#define unchar(ch)  ((ch) - ' ')
#define ctl(ch)     ((ch) ^ 64 )
#define unpar(ch)   ((ch) & 127)
#define check(s)    tochar ( ((s) + (((s) & 192) / 64)) & 63)

/* Global Variables */

static int size;                       /* Size of present data */
static int rpsiz;                      /* Maximum receive packet size */
static int spsiz;                      /* Maximum send packet size */
static int pad;                        /* How much padding to send */
static int timint;                     /* Timeout for foreign host on sends */
static int n;                          /* Packet number */
static int numtry;                     /* Times this packet retried */
static int oldtry;                     /* Times previous packet retried */

static int state;                      /* Present state of the automaton */
static int cchksum;                    /* Our (computed) checksum */
static int padchar;                    /* Padding character to send */
static int eol;                        /* End-Of-Line character to send */
static int escchr;                     /* Connect command escape character */
static int quote;                      /* Quote character in incoming data */
static char recpkt[MAXPACKSIZ];	/* Receive packet buffer */
static char packet[MAXPACKSIZ];	/* Packet buffer */

static int c_abort_key;
static int c_user_aborted;

static FILE *fp;                       /* File pointer for current disk file */
//static FILE *debug;

int sendsw(char *filename, char abort_key);
int sinit(void);
int sfile(char *filename);
int sdata(void);
int seof(void);
int sbreak(void);
int recsw(char abort_key);
int rinit(void);
int rfile(void);
int rdata(void);
int dopar(int ch);
void spack(int type, int num, int len, char *data);
int rpack(int *len, int *num, char *data);
int cinchr(void);
int inchr(void);
int bufill(char *buffer);
void bufemp(char *buffer, int len);
void spar(char *buf);
void rpar(char *buf);
void prerrpkt(char *data);

/*
 *  s e n d s w
 *
 *  Sendsw is the state table switcher for sending files.  It loops until
 *  either it finishes, or an error is encountered.  The routines called
 *  by sendsw are responsible for changing the state.
 *
 */
int sendsw(char *filename, char abort_key)
{
	c_abort_key = abort_key;	
	c_user_aborted = FALSE;
	
	state = 'S';				/* Send initiate is the start state */
	eol = '\r';
	n = 0;						/* Initialize message number */
	numtry = 0;					/* Say no tries yet */
	fp = NULL;

    while (1) {                /* Do this as long as necessary */
		switch (state) {
		case 'S':				/* Send-Init */
			state = sinit();
			break;
		case 'F':				/* Send-File */
			state = sfile(filename);
			break;
		case 'D':				/* Send-Data */
			state = sdata();
			break;
		case 'Z':				/* Send-End-of-File */
			state = seof();
			break;
		case 'B':				/* Send-Break */
			state = sbreak();
			break;
		case 'C':				/* Complete */
	        return (KERMIT_TRUE);
		case 'A':				/* "Abort" */
		default:				/* Unknown, fail */
			if(c_user_aborted)
				return -2;
            
			return (KERMIT_FALSE);
		}
	}
}


/*
 *  s i n i t
 *
 *  Send Initiate: send this host's parameters and get other side's back.
 */

int sinit(void)
{
	int num, len;				/* Packet number, length */


	if (numtry++ > MAXTRY)		/* If too many tries, give up */
		return ('A');
	
	spar(packet);				/* Fill up init info packet */

	spack('S', n, 6, packet);	/* Send an S packet */
	
	switch (rpack(&len, &num, recpkt)) {	/* What was the reply? */
	case 'N':					/* NAK, try it again */
		return (state);

	case 'Y':					/* ACK */
		if (n != num)			/* If wrong ACK, stay in S state */
			return (state);		/* and try again */
		rpar(recpkt);			/* Get other side's init info */

		if (eol == 0)			/* Check and set defaults */
			eol = '\r';
		if (quote == 0)
			quote = '#';

		numtry = 0;				/* Reset try counter */
		n = (n + 1) % 64;		/* Bump packet count */
		return ('F');			/* OK, switch state to F */

	case 'E':					/* Error packet received */
		prerrpkt(recpkt);		/* Print it out and */
		return ('A');			/* abort */

    case -2:
		c_user_aborted = TRUE;
		return KERMIT_FALSE;
	case KERMIT_FALSE:                /* Receive failure, try again */
		return (state);
	default:					/* Anything else, just "abort" */
		return ('A');
	}
}


/*
 *  s f i l e
 *
 *  Send File Header.
 */

int sfile(char *filename)
{
	int num, len;				/* Packet number, length */

	if (numtry++ > MAXTRY)		/* If too many tries, give up */
		return ('A');
	if (fp == NULL) {
		fp = fopen(filename, "rb");		/* open the file to be sent */
		if (fp == NULL) {		/* If bad file pointer, give up */
			len = sprintf(packet, "Cannot open file %s", filename);
			spack('E', n, len, packet);
			return ('A');
		}
	}
	len = strlen(filename);
	spack('F', n, len, filename);	/* Send an F packet */
	switch (rpack(&len, &num, recpkt)) {	/* What was the reply? */
	case 'N':					/* NAK, just stay in this state, */
		num = (--num < 0 ? 63 : num);	/* unless it's NAK for next packet */
		if (n != num)			/* which is just like an ACK for */
			return (state);		/* this packet so fall thru to... */

	case 'Y':					/* ACK */
		if (n != num)			/* If wrong ACK, stay in F state */
			return (state);
		numtry = 0;				/* Reset try counter */
		n = (n + 1) % 64;		/* Bump packet count */
		size = bufill(packet);	/* Get first data from file */
		return ('D');			/* Switch state to D */

	case 'E':					/* Error packet received */
		prerrpkt(recpkt);		/* Print it out and */
		return ('A');			/* abort */

    case -2:
		c_user_aborted = TRUE;
		return KERMIT_FALSE;
	case KERMIT_FALSE:
		return (state);			/* Receive failure, stay in F state */

	default:
		return ('A');			/* Something else, just "abort" */
	}
}


/*
 *  s d a t a                 getc
 *
 *  Send File Data
 */

int sdata(void)
{
	int num, len;				/* Packet number, length */
	unsigned char  rst;
	if (numtry++ > MAXTRY)		/* If too many tries, give up */
		return ('A');

	spack('D', n, size, packet);	/* Send a D packet */
	
	rst = rpack(&len, &num, recpkt);

	switch (rst) {	/* What was the reply? */
	case 'N':					/* NAK, just stay in this state, */
		num = (--num < 0 ? 63 : num);	/* unless it's NAK for next packet */
		if (n != num)			/* which is just like an ACK for */
			return (state);		/* this packet so fall thru to... */

	case 'Y':					/* ACK */
		if (n != num) {			/* If wrong ACK, fail */
			return (state);
		}
		numtry = 0;				/* Reset try counter */
		n = (n + 1) % 64;		/* Bump packet count */
		if ((size = bufill(packet)) == EOF)		/* Get data from file */
			return ('Z');		/* If EOF set state to that */
		return ('D');			/* Got data, stay in state D */

	case 'E':					/* Error packet received */
		prerrpkt(recpkt);		/* Print it out and */
		return ('A');			/* abort */

    case -2:
		c_user_aborted = TRUE;
		return KERMIT_FALSE;
	case KERMIT_FALSE:                /* Receive failure, stay in D */
		return (state);

	default:					/* Anything else, "abort" */
		return ('A');
	}
}


/*
 *  s e o f
 *
 *  Send End-Of-File.
 */

int seof(void)
{
	int num, len;				/* Packet number, length */
	if (numtry++ > MAXTRY)		/* If too many tries, "abort" */
		return ('A');

	spack('Z', n, 0, NULL);		/* Send a 'Z' packet */
	switch (rpack(&len, &num, recpkt)) {	/* What was the reply? */
	case 'N':					/* NAK, just stay in this state, */
		num = (--num < 0 ? 63 : num);	/* unless it's NAK for next packet, */
		if (n != num)			/* which is just like an ACK for */
			return (state);		/* this packet so fall thru to... */

	case 'Y':					/* ACK */
		if (n != num)
			return (state);		/* If wrong ACK, hold out */
		numtry = 0;				/* Reset try counter */
		n = (n + 1) % 64;		/* and bump packet count */
		fclose(fp);				/* Close the input file */
		fp = NULL;				/* Set flag indicating no file open */

		return ('B');			/* EOT, all done */

	case 'E':					/* Error packet received */
		prerrpkt(recpkt);		/* Print it out and */
		return ('A');			/* abort */

    case -2:
		c_user_aborted = TRUE;
		return KERMIT_FALSE;
	case KERMIT_FALSE:                /* Receive failure, stay in Z */
		return (state);

	default:
		return ('A');			/* Something else, "abort" */
	}
}


/*
 *  s b r e a k
 *
 *  Send Break (EOT)
 */

int sbreak(void)
{
	int num, len;				/* Packet number, length */
	if (numtry++ > MAXTRY)		/* If too many tries "abort" */
		return ('A');

	spack('B', n, 0, NULL);		/* Send a B packet */
	switch (rpack(&len, &num, recpkt)) {	/* What was the reply? */
	case 'N':					/* NAK, just stay in this state, */
		num = (--num < 0 ? 63 : num);	/* unless NAK for previous packet, */
		if (n != num)			/* which is just like an ACK for */
			return (state);		/* this packet so fall thru to... */

	case 'Y':					/* ACK */
		if (n != num)
			return (state);		/* If wrong ACK, fail */
		numtry = 0;				/* Reset try counter */
		n = (n + 1) % 64;		/* and bump packet count */
		return ('C');			/* Switch state to Complete */

	case 'E':					/* Error packet received */
		prerrpkt(recpkt);		/* Print it out and */
		return ('A');			/* abort */

    case -2:
		c_user_aborted = TRUE;
		return KERMIT_FALSE;
	case KERMIT_FALSE:
		return (state);			/* Receive failure, stay in B */

	default:
		return ('A');			/* Other, "abort" */
	}
}


/*
 *  r e c s w
 *
 *  This is the state table switcher for receiving files.
 */

int recsw(char abort_key)
{
	c_abort_key = abort_key;	
	c_user_aborted = FALSE;
		
	state = 'R';				/* Receive-Init is the start state */
	n = 0;						/* Initialize message number */
	numtry = 0;					/* Say no tries yet */

    while (1) {
		//printf("state=%c\n", state);
		switch (state) {		/* Do until done */
		case 'R':				/* Receive-Init */
			state = rinit();
			break;
		case 'F':				/* Receive-File */
			state = rfile();
			break;
		case 'D':				/* Receive-Data */
			state = rdata();
			break;
		case 'C':				/* Complete state */
            return (KERMIT_TRUE);
		case 'A':				/* "Abort" state */
	    	if(c_user_aborted)
				return -2;
			
			return (KERMIT_FALSE);
		}
	}
}


/*
 *  r i n i t
 *
 *  Receive Initialization
 */

int rinit(void)
{
	int len, num;				/* Packet length, number */

	if (numtry++ > MAXTRY)		/* If too many tries, "abort" */
		return ('A');

	switch (rpack(&len, &num, recpkt)) {	/* Get a packet */
	case 'S':					/* Send-Init */
		rpar(recpkt);			/* Get the other side's init data */
		spar(packet);			/* Fill up packet with my init info */
		spack('Y', n, 6, packet);	/* ACK with my parameters */
		oldtry = numtry;		/* Save old try count */
		numtry = 0;				/* Start a new counter */
		n = (n + 1) % 64;		/* Bump packet number, mod 64 */
		return ('F');			/* Enter File-Receive state */

	case 'E':					/* Error packet received */
		prerrpkt(recpkt);		/* Print it out and */
		return ('A');			/* abort */

    case -2:
		c_user_aborted = TRUE;
		return KERMIT_FALSE;
	case KERMIT_FALSE:          /* Didn't get packet */
		spack('N', n, 0, NULL);	/* Return a NAK */
		return (state);			/* Keep trying */

	default:
		return ('A');			/* Some other packet type, "abort" */
	}
}


/*
 *  r f i l e
 *
 *  Receive File Header
 */

int rfile(void)
{
	int num, len;				/* Packet number, length */
	char filename[20];			/* Holds the converted file name */

	if (numtry++ > MAXTRY)		/* "abort" if too many tries */
		return ('A');

	switch (rpack(&len, &num, recpkt)) {	/* Get a packet */
	case 'S':					/* Send-Init, maybe our ACK lost */
		if (oldtry++ > MAXTRY)	/* If too many tries "abort" */
			return ('A');
		if (num == ((n == 0) ? 63 : n - 1)) {	/* Previous packet, mod 64? */
			/* Yes, ACK it again with  */
			spar(packet);		/* our Send-Init parameters */
			spack('Y', num, 6, packet);
			numtry = 0;			/* Reset try counter */
			return (state);		/* Stay in this state */
		} else
			return ('A');		/* Not previous packet, "abort" */

	case 'Z':					/* End-Of-File */
		if (oldtry++ > MAXTRY)
			return ('A');
		if (num == ((n == 0) ? 63 : n - 1)) {	/* Previous packet, mod 64? */
			/* Yes, ACK it again. */
			spack('Y', num, 0, NULL);
			numtry = 0;
			return (state);		/* Stay in this state */
		} else
			return ('A');		/* Not previous packet, "abort" */

	case 'F':					/* File Header (just what we want) */
		if (num != n)			/* The packet number must be right */
			return ('A');
		strcpy(filename, recpkt);	/* Copy the file name */


		if ((fp = fopen(filename, "wb")) == NULL) {		/* Try to open a new file */
			len = sprintf(packet, "Cannot create %s", filename);	/* Give up if can't */
			spack('E', n, len, packet);
			return ('A');
		}
		spack('Y', n, 0, NULL);	/* Acknowledge the file header */
		oldtry = numtry;		/* Reset try counters */
		numtry = 0;				/* ... */
		n = (n + 1) % 64;		/* Bump packet number, mod 64 */
		return ('D');			/* Switch to Data state */

	case 'B':					/* Break transmission (EOT) */
		if (num != n)			/* Need right packet number here */
			return ('A');
		spack('Y', n, 0, NULL);	/* Say OK */
		return ('C');			/* Go to complete state */

	case 'E':					/* Error packet received */
		prerrpkt(recpkt);		/* Print it out and */
		return ('A');			/* abort */

    case -2:
		c_user_aborted = TRUE;
		return KERMIT_FALSE;
	case KERMIT_FALSE:          /* Didn't get packet */
		spack('N', n, 0, NULL);	/* Return a NAK */
		return (state);			/* Keep trying */

	default:					/* Some other packet, "abort" */
		return ('A');
	}
}


/*
 *  r d a t a
 *
 *  Receive Data
 */

int rdata(void)
{
	int num, len;				/* Packet number, length */
	if (numtry++ > MAXTRY)		/* "abort" if too many tries */
		return ('A');

	switch (rpack(&len, &num, recpkt)) {	/* Get packet */
	case 'D':					/* Got Data packet */
		if (num != n) {			/* Right packet? */
			/* No */
			if (oldtry++ > MAXTRY)
				return ('A');	/* If too many tries, abort */
			if (num == ((n == 0) ? 63 : n - 1)) {	/* Else check packet number */
				/* Previous packet again? */
				spack('Y', num, 6, packet);		/* Yes, re-ACK it */
				numtry = 0;		/* Reset try counter */
				return (state);	/* Don't write out data! */
			} else
				return ('A');	/* sorry, wrong number */
		}
		/* Got data with right packet number */
		bufemp(recpkt, len);	/* Write the data to the file */
		spack('Y', n, 0, NULL);	/* Acknowledge the packet */
		oldtry = numtry;		/* Reset the try counters */
		numtry = 0;				/* ... */
		n = (n + 1) % 64;		/* Bump packet number, mod 64 */
		return ('D');			/* Remain in data state */

	case 'F':					/* Got a File Header */
		if (oldtry++ > MAXTRY)
			return ('A');		/* If too many tries, "abort" */
		if (num == ((n == 0) ? 63 : n - 1)) {	/* Else check packet number */
			/* It was the previous one */
			spack('Y', num, 0, NULL);	/* ACK it again */
			numtry = 0;			/* Reset try counter */
			return (state);		/* Stay in Data state */
		} else					/* Not previous packet, "abort" */
			return ('A');

	case 'Z':					/* End-Of-File */
		if (num != n)			/* Must have right packet number */
			return ('A');
		spack('Y', n, 0, NULL);	/* OK, ACK it. */
		fclose(fp);				/* Close the file */
		n = (n + 1) % 64;		/* Bump packet number */
		return ('F');			/* Go back to Receive File state */

	case 'E':					/* Error packet received */
		prerrpkt(recpkt);		/* Print it out and */
		return ('A');			/* abort */

	case -2:
		c_user_aborted = TRUE;
		return KERMIT_FALSE;
    case KERMIT_FALSE:                /* Didn't get packet */
		spack('N', n, 0, NULL);	/* Return a NAK */
		return (state);			/* Keep trying */

	default:
		return ('A');			/* Some other packet, "abort" */
	}
}


/*
 *      KERMIT utilities.
 */


void prerrpkt(char *data)
{
	printf("ERROR: |%s|\n", data);
}


void spar(char *buf)
{
	sprintf(buf, "%c%c%c%c%c%c",
			tochar(MAXPACKSIZ),
			tochar(MYTIME),
			tochar(MYPAD),
			ctl(MYPCHAR),
			tochar(MYEOL),
			MYQUOTE);
}

void rpar(char *buf)
{
	rpsiz = unchar(buf[0]);
	spsiz = MAXPACKSIZ;
	if (rpsiz < spsiz)
		spsiz = rpsiz;

	timint=unchar(buf[1]);
	pad = unchar(buf[2]);
	padchar = ctl(buf[3]);
	eol = unchar(buf[4]);
	quote = buf[5];
	//printf("\flen=%d\neol=%X\nquote=%c", rpsiz, eol, quote);
}

/*
 *  s p a c k
 *
 *  Send a Packet
 */
void spack(int type, int num, int len, char *data)
{
	int i, n;					/* Character loop counter */
	int chksum;
	char buffer[MAXPACKSIZ + 6];	/* Checksum, packet buffer */

    //fputs("\nspack\n",debug);
	for (i = 1; i <= pad; i++)	/* Issue any padding */
		put_comF(padchar, c_abort_key);
	i = 0;
	chksum = 0;
	buffer[i++] = SOH;			/* Packet marker, ASCII 1 (SOH) */
	buffer[i++] = tochar(len + 3);	/* Send the character count */
	buffer[i++] = tochar(num);	/* Packet number */
	buffer[i++] = type;			/* Packet type */
	strcpy(buffer + i, data);
	i += len;
	for (n = 1; n < i; n++)
		chksum += buffer[n];
	buffer[i++] = check(chksum);	/* Put it in the packet */
	buffer[i++] = eol;			/* Extra-packet line terminator */

    transmit_string_ir(buffer,i, c_abort_key);
}

/*
 *  r p a c k
 *
 *  Read a Packet
 */
int rpack(int *len, int *num, char *data)
{
	int i, done;				/* Data character number, loop exit */
	int t;						/* Current input character */
	int type;					/* Packet type */
	int rchksum;				/* Checksum received from other host */

	cchksum = 0;
	if ((timint > MAXTIM) || (timint < MINTIM))
		timint = MYTIME;
	t = -1;
	
	while (t != SOH)  {			/* Wait for packet header */
		if ((t = get_comF(timint * 50, c_abort_key)) < 0)
			return (t);
	}
	
	done = KERMIT_FALSE;              /* Got SOH, init loop */
	while (done) {				/* Loop to get a packet */
		cchksum = 0;
		if ((t = get_comF(timint * 50, c_abort_key)) < 0)
			return (t);
		if (t == SOH) {			/* Resynchronize if SOH */
			continue;
		}
		cchksum += t;
		*len = unchar(t) - 3;	/* Character count */
		if ((t = get_comF(timint * 50, c_abort_key)) < 0)
			return (t);
		if (t == SOH)			/* Resynchronize if SOH */
			continue;
		cchksum += t;
		*num = unchar(t);		/* Packet number */
		if ((t = get_comF(timint * 50, c_abort_key)) < 0)
			return (t);
		if (t == SOH)			/* Resynchronize if SOH */
			continue;
		cchksum += t;
		type = t;				/* Packet type */

/* Put len characters from the packet into the data buffer */

		for (i = 0; i < *len; i++) {
			if ((t = get_comF(timint * 50, c_abort_key)) < 0)
				return (t);
			if ((data[i] = t) == SOH)
				break;			/* Resynch if SOH */
			cchksum += t;
		}
		if (t == SOH)			/* Resynch if SOH */
			continue;
		data[*len] = 0;			/* Mark the end of the data */
		if ((t = get_comF(timint * 50, c_abort_key)) < 0)
			return (t);
		if (t == SOH)			/* Resynchronize if SOH */
			continue;
		rchksum = t;			/* Convert to numeric */
		done = KERMIT_TRUE;              /* Got checksum, done */
	}
	if (check(cchksum) != rchksum) {
		//printf("DEBUG: checksum error\n");
		return (t);
	}
	
	return (type);				/* All OK, return packet type */
}

/*
 *  b u f i l l
 *
 *  Get a bufferful of data from the file that's being sent.
 *  Only control-quoting is done; 8-bit & repeat count prefixes are
 *  not handled.
 */

int bufill(char *buffer)
{
	int i;						/* Loop index */
	int t;						/* Char read from file */
	char t7;					/* 7-bit version of above */

	i = 0;						/* Init data buffer pointer */
	while ((t = fgetc(fp)) != EOF) {	/* Get the next character */
		t7 = unpar(t);			/* Get low order 7 bits */
		if (t7 < SP || t7 == DEL || t7 == quote) {	/* Does this char require */
			/* special handling? */
			buffer[i++] = quote;	/* Quote the character */
			if (t7 != quote) {
				t = ctl(t);         /* and uncontrolify */
				t7 = ctl(t7);
			}
		}
		if (i >= spsiz-8) {
			i--;
		//	printf(" spsize = %d ",(int)spsiz);
			ungetc(ctl(t),fp);
			return(i);
		}
		buffer[i++] = t;		/* Deposit the character itself */
		if (i >= spsiz - 8) {     /* Check length */
		//	printf(" spsize2 = %d ",(int)spsiz);
			return (i);
		}
	}
	if (i == 0)					/* Wind up here only on EOF */
		return (EOF);
	//printf(" spsize3 = %d ",(int)spsiz);
	return (i);					/* Handle partial buffer */
}


/*
 *      b u f e m p
 *
 *  Put data from an incoming packet into a file.
 */

void bufemp(char *buffer, int len)
{
	int i;						/* Counter */
	char t;						/* Character holder */

	for (i = 0; i < len;) {
		t = *buffer++;
		i++;
		if (t == quote) {
			t = *buffer++;
			i++;
			if (t != quote)
				t = ctl(t);
			fputc(t, fp);
		} else
			fputc(t, fp);
	}
}

